unit SearchUnit;

interface

uses
	SysUtils, Classes, QGraphics, QControls, QForms, QDialogs,
	QStdCtrls, QExtCtrls, QComCtrls;

type
  TSearchForm = class(TForm)
    Label1: TLabel;
    SearchStringEdit: TEdit;
    Bevel1: TBevel;
    CloseButton: TButton;
    IncludeBodyCheckBox: TCheckBox;
    StartButton: TButton;
		ResultsList: TListView;
    Label2: TLabel;
    AuthorEdit: TEdit;
    Label3: TLabel;
    FirstDateEdit: TEdit;
    Label4: TLabel;
    LastDateEdit: TEdit;
		procedure StartButtonClick(Sender: TObject);
		procedure ResultsListDblClick(Sender: TObject);
		procedure ResultsListKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
	private
    FSearching: Boolean;
    procedure SetSearching(const Value: Boolean);
	public
		property Searching: Boolean read FSearching write SetSearching;
	end;

implementation

{$R *.dfm}

uses
	UtilsWin,
	TopicUnit, TopicContentsUnit,
	MainUnit;

procedure TSearchForm.StartButtonClick(Sender: TObject);
var
	I,
	J,
	K: Integer;
	IncludeBody,
	CheckAuthor,
	CheckDate,
	ContainsWord,
	ContainsAllWords: Boolean;
	Author: string;
	FirstDate,
	LastDate: TDateTime;
function IsValidAuthor(const UserID: string): Boolean;
begin
	Result := (not CheckAuthor) or (Pos (Author, LowerCase (UserID)) > 0) or (Pos (Author, LowerCase (MainForm.Aliases.Values [UserID])) > 0);
end;
function IsValidDate(const PostDate: TDateTime): Boolean;
begin
	Result := (not CheckDate) or ((PostDate >= FirstDate) and (PostDate < LastDate));
end;
begin
	Searching := not Searching;
	if Searching then begin
		with ResultsList.Items do begin
			BeginUpdate;
			try
				Clear;
			finally
				EndUpdate;
			end;
		end;
		IncludeBody := IncludeBodyCheckBox.Checked;
		with MainForm.SearchWords do begin
			CommaText := SearchStringEdit.Text;
			for J := 0 to Count - 1 do
				Strings [J] := LowerCase (Strings [J]);
			SearchStringEdit.Text := StringReplace (CommaText, ',', ', ', [rfReplaceAll]);
		end;
		Author := LowerCase (AuthorEdit.Text);
		CheckAuthor := Length (Author) > 0;
		FirstDate := 0;
		if Length (FirstDateEdit.Text) > 0 then try
			FirstDate := StrToDateTime (FirstDateEdit.Text);
		except
			FirstDateEdit.Text := '';
		end;
		LastDate := 0;
		if Length (LastDateEdit.Text) > 0 then try
			LastDate := StrToDateTime (LastDateEdit.Text);
			if Pos (':', LastDateEdit.Text) <= 0 then
				LastDate := LastDate + 1;
		except
			LastDateEdit.Text := '';
		end;
		CheckDate := (FirstDate <> 0) and (LastDate <> 0);
		with MainForm, TopicList.Items do begin
			if TopicList.SortDirection = sdDescending then
				I := 0
			else
				I := Count - 1;
			while (I >= 0) and (I < Count) do begin
				with Item [I] do
					if Assigned (Data) then begin
						ContainsAllWords := True;
						with TTopic (Data) do begin
							if (not IncludeBody) and (not (IsValidDate (Started) and IsValidAuthor (StartedBy))) then begin
								if TopicList.SortDirection = sdDescending then
									Inc (I)
								else
									Dec (I);
								Continue;
							end;
							if SearchWords.Count > 0 then begin
								for J := 0 to SearchWords.Count - 1 do
									if (not (IsValidDate (Started) and IsValidAuthor (StartedBy))) or (Pos (SearchWords.Strings [J], LowerCase (TopicName)) <= 0) then begin
										if IncludeBody then begin
											ContainsWord := False;
											with TopicContents do
												for K := 0 to Count - 1 do
													with TTopicPost (Items [K]) do
														if IsValidDate (PostTime) and IsValidAuthor (UserID) and ((Pos (SearchWords.Strings [J], LowerCase (Title)) > 0) or (Pos (SearchWords.Strings [J], LowerCase (Text)) > 0)) then begin
															ContainsWord := True;
															Break;
														end;
											if not ContainsWord then begin
												ContainsAllWords := False;
												Break;
											end;
										end else begin
											ContainsAllWords := False;
											Break;
										end;
									end;
							end else begin
								ContainsWord := False;
								with TopicContents do
									for K := 0 to Count - 1 do
										with TTopicPost (Items [K]) do
											if IsValidDate (PostTime) and IsValidAuthor (UserID) then begin
												ContainsWord := True;
												Break;
											end;
								if not ContainsWord then
									ContainsAllWords := False;
							end;
						end;
						if ContainsAllWords then
							with ResultsList.Items.Add do begin
								Data := Item[I].Data;
								ImageIndex := Item[I].ImageIndex;
								Caption := Item[I].Caption;
								SubItems.Assign (Item[I].SubItems);
								Application.ProcessMessages;
								if not Searching then
									Exit;
							end;
					end;
				if I mod 16 = 0 then begin
					Application.ProcessMessages;
					if not Searching then
						Exit;
				end;
				if TopicList.SortDirection = sdDescending then
					Inc (I)
				else
					Dec (I);
			end;
		end;
		Searching := False;
		if ResultsList.Items.Count <= 0 then
			ShowDefaultMessageBox ('No matching topics found.', 'Error', mtInformation);
	end;
end;

procedure TSearchForm.ResultsListDblClick(Sender: TObject);
begin
	with ResultsList do
		if Assigned (Selected) then
			with Selected do
				if Assigned (Data) then
					with TTopic (Data) do
						Show;
end;

procedure TSearchForm.ResultsListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
	case Key of
		13:
			ResultsListDblClick (Sender);
	end;
end;

procedure TSearchForm.SetSearching(const Value: Boolean);
begin
	if FSearching <> Value then begin
		FSearching := Value;
		if Value then
			StartButton.Caption := '&Stop'
		else
			StartButton.Caption := '&Start';
	end;
end;

procedure TSearchForm.FormCreate(Sender: TObject);
begin
	ResultsList.Columns.Assign (MainForm.TopicList.Columns);
end;

procedure TSearchForm.CloseButtonClick(Sender: TObject);
begin
	Searching := False;
	ModalResult := mrCancel;
end;

end.
