unit MainUnit;

interface

uses
	MasterUnit, TopicUnit,
	SettingsUnit,
	blcksock,
	SysUtils, Classes, QGraphics, QControls, QForms, QExtCtrls,
	QMenus, QTypes, QImgList, QStdCtrls, QComCtrls;

const
	InstantUpdates = False;
	PrefsFile = '.boardreaderrc';
	ListFile = 'list.brl';

type
	TExecMethod = procedure of object;

	TMainForm = class(TForm)
		TopicList: TListView;
		StatusBar: TStatusBar;
		TopicIcons: TImageList;
		TopicListPopupMenu: TPopupMenu;
		MarkAsReadMenuItem: TMenuItem;
		MarkAsUnreadMenuItem: TMenuItem;
		TopicDownloadTimer: TTimer;
		N1: TMenuItem;
		RedownloadMenuItem: TMenuItem;
		StopDownloadingMenuItem: TMenuItem;
		N2: TMenuItem;
		RefreshMenuItem: TMenuItem;
		RefreshAllMenuItem: TMenuItem;
		URLPanel: TPanel;
		Label1: TLabel;
		ButtonPanePanel: TPanel;
		ButtonPanel: TPanel;
		SearchButton: TButton;
		SettingsButton: TButton;
		NewTopicButton: TButton;
		UpdateTimer: TTimer;
		URLEdit: TComboBox;
		AliasesButton: TButton;
		WhosOnlineIndicator: TLabel;
    ExecProcTimer: TTimer;
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure AppException(Sender: TObject; E: Exception);
		procedure TopicListDeletion(Sender: TObject; Item: TListItem);
		procedure TopicListDblClick(Sender: TObject);
		procedure SearchButtonClick(Sender: TObject);
		procedure TopicListSelectItem(Sender: TObject; Item: TListItem;
			Selected: Boolean);
		procedure MarkAsReadMenuItemClick(Sender: TObject);
		procedure MarkAsUnreadMenuItemClick(Sender: TObject);
		procedure URLEditKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure FormKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure FormClose(Sender: TObject; var Action: TCloseAction);
		procedure TopicDownloadTimerTick(Sender: TObject);
		procedure TopicListKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure RedownloadMenuItemClick(Sender: TObject);
		procedure StopDownloadingMenuItemClick(Sender: TObject);
		procedure RefreshMenuItemClick(Sender: TObject);
		procedure RefreshAllMenuItemClick(Sender: TObject);
		procedure SettingsButtonClick(Sender: TObject);
		procedure NewTopicButtonClick(Sender: TObject);
		procedure UpdateTimerTick(Sender: TObject);
		procedure FormActivate(Sender: TObject);
		procedure URLEditSelect(Sender: TObject);
		procedure TopicListEditing(Sender: TObject; Item: TListItem;
			var AllowEdit: Boolean);
		procedure TopicListClick(Sender: TObject);
		procedure AliasesButtonClick(Sender: TObject);
		procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
		procedure TopicListEdited(Sender: TObject; Item: TListItem;
			var S: WideString);
		procedure TopicListColumnClick(Sender: TObject; Column: TListColumn);
    procedure ExecProcTimerTick(Sender: TObject);
	private
		FBoardName: string;
		FConnectionStatus: TConnectionStatus;
		FURL: string;
		FStatusText: string;
		FDownloadTopic: TTopic;
		FCurIP: string;
		FCurHost: string;
		ExecProcArray: array of TExecMethod;
		procedure SetBoardName(const Value: string);
		procedure SetConnectionStatus(const Value: TConnectionStatus);
		procedure SetURL(Value: string);
		procedure SetStatusText(const Value: string);
		procedure SetDownloadTopic(const Value: TTopic);
		function GetURLPath: string;
		function GetURLID: string;
	public
		FirstStart: Boolean;
		HomeDir: string;
		Client: TTCPBlockSocket;
		UseProxy: Boolean;
		Proxy: string;
		ProxyPort: Integer;
		GlobalCookie,
		LocalCookie: string;
		TextToSend: string;
		TextReceived: string;
		CompleteDoc: Boolean;
		SearchWords,
		Aliases: TStringList;
		MainColumn,
		TopicColumn,
		RepliesColumn,
		StartedByColumn,
		LastCommentColumn,
		StartedColumn,
		LatestChangeColumn: TListColumn;
		CurrentAction: TRequestAction;
		UpdatingPopup,
		UpdatingPostCount,
		InNewRequest: Boolean;
		ProgramStarted,
		Closing,
		Closed: Boolean;
		DownloadURL,
		PageURLs: string;
		TopicsToDownload,
		WaitBeforeDownload: Integer;
		TotalPosts: Integer;
		ShowAfterDownload,
		NextDownloadsNotAnonymous: Boolean;
		OwnID: string;
		NewTopicForm: TForm;
		PostExtras: TPostExtras;
		EmailReplies: Boolean;
		ShouldRequestUpdates,
		ShouldSetURL,
		HideNextUpdate,
		HideInitUpdates,
		HideAutoUpdates,
		HideManualUpdates,
		HideTopicDownloads,
		MarkForumsRead: Boolean;
		SettingsForm: TSettingsForm;
		TopicWinLeft,
		TopicWinTop,
		TopicWinWidth,
		TopicWinHeight: Integer;
		ButtonsVertical,
		ButtonsLeft,
		ButtonsTop: Boolean;
		CanEditOwn,
		CanEditAll,
		CanDeleteOwn,
		CanDeleteAll: Boolean;
		NewPostCount,
		InErrorCount: Integer;
		TimeCorrection: Integer;
		TimeCorrectionChanged: Boolean;
		procedure ExecProc(Method: TExecMethod);
		procedure LoadSettings;
		procedure SaveSettings;
		procedure LoadTopics;
		procedure SaveTopics;
		procedure LoadAliases;
		procedure SaveAliases;
		procedure LoadFavorites;
		procedure UpdateAliases;
		procedure MakeReady;
		procedure SendCommand(const HostName, Command: string; const CommandURL: string = ''; const Header: string = ''; const Body: string = '');
		procedure DocReceived;
		procedure CancelAction;
		procedure GlobalLogin(const UserName, Password: string);
		procedure LocalLogin(const UserName, Password, BoardName: string);
		procedure RequestUpdates(Hidden: Boolean = False);
		procedure RequestAll;
		procedure RequestTopicListPage;
		procedure RequestListDocReceived;
		procedure RequestNextTopic(Hidden: Boolean = False);
		procedure RequestTopic(Topic: TTopic; Hidden: Boolean = False);
		procedure RequestTopicDocReceived;
		procedure ReloadPostDocReceived;
		procedure AddSubItem(ListItem: TListItem; Column: TListColumn; const S: string);
		function FindTopicByURL(AURL: string): TListItem;
		procedure UpdatePopupMenu;
		procedure UpdatePostCount;
		procedure UpdateButtonPositions;
		function GetCookieHeader: string;
		procedure ProcessNewCookies(Doc: string);
		procedure SetCookie(var CookieStr: string; const NewCookie: string);
		procedure CheckForCookie;
		procedure DeleteSelectedTopics;
		procedure ClearFavorites;
		procedure AddFavorite(const FavName, FavURL: string);
		function GetAlias(const UserID: string): string;
		property URL: string read FURL write SetURL;
		property URLID: string read GetURLID;
		property URLPath: string read GetURLPath;
		property BoardName: string read FBoardName write SetBoardName;
		property CurHost: string read FCurHost write FCurHost;
		property CurIP: string read FCurIP write FCurIP;
		property ConnectionStatus: TConnectionStatus read FConnectionStatus write SetConnectionStatus;
		property StatusText: string read FStatusText write SetStatusText;
		property DownloadTopic: TTopic read FDownloadTopic write SetDownloadTopic;
	end;

var
	MainForm: TMainForm;

	HasBeenShown: Boolean = False;

implementation

{$R *.dfm}

uses
	IniFiles,
	MainAidUnit,
	FileIOUnit, TopicContentsUnit,
	SearchUnit, AppendForumUnit, TopicWinUnit, AliasesUnit,
	UtilsSys, UtilsWin;

{ TMainForm }

procedure TMainForm.ExecProc(Method: TExecMethod);
begin
	SetLength (ExecProcArray, Length (ExecProcArray) + 1);
	ExecProcArray [High (ExecProcArray)] := Method;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
	I: Integer;
	Column: TListColumn;
	Title: string;
begin
	Application.OnException := AppException;
	HomeDir := GetEnvironmentVariable ('HOME');
	if (Length (HomeDir) > 0) and (HomeDir [Length (HomeDir)] <> '/') then
		HomeDir := HomeDir + '/';
	SearchWords := TStringList.Create;
	Aliases := TStringList.Create;
	Aliases.Sorted := True;
	MainColumn := TopicList.Columns.Items [0];
	with TopicList.Columns do
		for I := 0 to Count - 1 do begin
			Column := Items [I];
			Title := UpperCase (Column.Caption);
			if Title = 'TOPIC' then
				TopicColumn := Column
			else if Title = 'REPLIES' then
				RepliesColumn := Column
			else if Title = 'STARTED BY' then
				StartedByColumn := Column
			else if Title = 'LAST COMMENT' then
				LastCommentColumn := Column
			else if Title = 'STARTED' then
				StartedColumn := Column
			else if Title = 'LATEST CHANGE' then
				LatestChangeColumn := Column;
		end;
	TopicList.SortColumn := LatestChangeColumn.Index;
	HideAutoUpdates := True;
	MarkForumsRead := True;
	PostExtras := [peAutoLinks];
	LoadSettings;
	LoadAliases;
	LoadFavorites;
	HideNextUpdate := HideAutoUpdates;
	TopicList.Sorted := True;
	if ParamCount >= 1 then begin
		URL := ParamStr (1);
		URLEdit.Text := URL;
		ActiveControl := TopicList;
	end;
	UpdatePostCount;
	URLEdit.Width := URLPanel.ClientWidth - 71;
	WhosOnlineIndicator.Left := URLEdit.Left + URLEdit.Width + 6;
	Application.HintHidePause := 10000;
	ProgramStarted := True;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
	Aliases.Free;
	SearchWords.Free;
end;

procedure TMainForm.LoadSettings;
var
	I: Integer;
begin
	if FileExists (HomeDir + PrefsFile) then begin
		with TMemIniFile.Create (HomeDir + PrefsFile) do try
			UseProxy := ReadBool ('Settings', 'Use Proxy', False);
			Proxy := ReadString ('Settings', 'Proxy', '');
			ProxyPort := ReadInteger ('Settings', 'Proxy Port', 0);
			GlobalCookie := ReadString ('Settings', 'Global Cookie', '');
			TopicList.SortColumn := ReadInteger ('Settings', 'Sort Column', 0);
			if ReadBool ('Settings', 'Sort Backwards', False) then
				TopicList.SortDirection := sdDescending
			else
				TopicList.SortDirection := sdAscending;
			with TopicList.Columns do begin
				for I := 0 to Count - 1 do
					if ValueExists ('Settings', 'Column Width ' + IntToStr (I + 1)) then
						Items[I].Width := ReadInteger ('Settings', 'Column Width ' + IntToStr (I + 1), 50);
			end;
			UpdateTimer.Enabled := ReadBool ('Settings', 'Periodic Update Check', False);
			UpdateTimer.Interval := ReadInteger ('Settings', 'Periodic Update Check Interval', 600000);
			HideInitUpdates := ReadBool ('Settings', 'Hide Initial Updates', False);
			HideAutoUpdates := ReadBool ('Settings', 'Hide Automatic Updates', True);
			HideManualUpdates := ReadBool ('Settings', 'Hide Manual Updates', False);
			HideTopicDownloads := ReadBool ('Settings', 'Hide Topic Downloads', True);
			MarkForumsRead := ReadBool ('Settings', 'Mark Forums Read', True);
			PostExtras := [];
			if ReadBool ('Settings', 'Use EZCodes', False) then
				Include (PostExtras, peEZCodes);
			if ReadBool ('Settings', 'Use Emoticons', False) then
				Include (PostExtras, peEmoticons);
			if ReadBool ('Settings', 'Use Link Conversion', True) then
				Include (PostExtras, peAutoLinks);
			if ReadBool ('Settings', 'Use HTML', False) then
				Include (PostExtras, peHTML);
			EmailReplies := ReadBool ('Settings', 'Email Reply Notification', False);
			if ValueExists ('Settings', 'Topic Win Left') then
				TopicWinLeft := ReadInteger ('Settings', 'Topic Win Left', 10);
			if ValueExists ('Settings', 'Topic Win Top') then
				TopicWinTop := ReadInteger ('Settings', 'Topic Win Top', 10);
			if ValueExists ('Settings', 'Topic Win Width') then
				TopicWinWidth := ReadInteger ('Settings', 'Topic Win Width', 620);
			if ValueExists ('Settings', 'Topic Win Height') then
				TopicWinHeight := ReadInteger ('Settings', 'Topic Win Height', 460);
			ButtonsVertical := ReadBool ('Settings', 'Buttons Vertical', False);
			ButtonsLeft := ReadBool ('Settings', 'Buttons Left', False);
			ButtonsTop := ReadBool ('Settings', 'Buttons Top', False);
		finally
			Free;
		end;
	end else
		FirstStart := True;
	UpdateButtonPositions;
end;

procedure TMainForm.SaveSettings;
var
	I: Integer;
begin
	if not Closed then begin
		with TMemIniFile.Create (HomeDir + PrefsFile) do try
			WriteBool ('Settings', 'Use Proxy', UseProxy);
			WriteString ('Settings', 'Proxy', Proxy);
			WriteInteger ('Settings', 'Proxy Port', ProxyPort);
			WriteString ('Settings', 'Global Cookie', GlobalCookie);
			WriteInteger ('Settings', 'Sort Column', TopicList.SortColumn);
			WriteBool ('Settings', 'Sort Backwards', TopicList.SortDirection = sdDescending);
			with TopicList.Columns do begin
				for I := 0 to Count - 1 do
					WriteInteger ('Settings', 'Column Width ' + IntToStr (I + 1), Items[I].Width);
			end;
			WriteBool ('Settings', 'Periodic Update Check', UpdateTimer.Enabled);
			WriteInteger ('Settings', 'Periodic Update Check Interval', UpdateTimer.Interval);
			WriteBool ('Settings', 'Hide Initial Updates', HideInitUpdates);
			WriteBool ('Settings', 'Hide Automatic Updates', HideAutoUpdates);
			WriteBool ('Settings', 'Hide Manual Updates', HideManualUpdates);
			WriteBool ('Settings', 'Hide Topic Downloads', HideTopicDownloads);
			WriteBool ('Settings', 'Mark Forums Read', MarkForumsRead);
			WriteBool ('Settings', 'Use EZCodes', peEZCodes in PostExtras);
			WriteBool ('Settings', 'Use Emoticons', peEmoticons in PostExtras);
			WriteBool ('Settings', 'Use Link Conversion', peAutoLinks in PostExtras);
			WriteBool ('Settings', 'Use HTML', peHTML in PostExtras);
			WriteBool ('Settings', 'Email Reply Notification', EmailReplies);
			WriteInteger ('Settings', 'Topic Win Left', TopicWinLeft);
			WriteInteger ('Settings', 'Topic Win Top', TopicWinTop);
			WriteInteger ('Settings', 'Topic Win Width', TopicWinWidth);
			WriteInteger ('Settings', 'Topic Win Height', TopicWinHeight);
			WriteBool ('Settings', 'Buttons Vertical', ButtonsVertical);
			WriteBool ('Settings', 'Buttons Left', ButtonsLeft);
			WriteBool ('Settings', 'Buttons Top', ButtonsTop);
			UpdateFile;
		finally
			Free;
		end;
	end;
end;

procedure TMainForm.LoadTopics;
var
	Stream: TFileStream;
	S: string;
	EndByte: Byte;
	Topic: TTopic;
	TotalReplies: Integer;
begin
	S := URLPath;
	if Length (S) > 0 then begin
		try
			if FileExists (S + ListFile) then begin
				Stream := TFileStream.Create (S + ListFile, fmOpenRead or fmShareDenyWrite);
				try
					SetLength (S, Length (FileIdent));
					try
						Stream.ReadBuffer (S [1], Length (FileIdent));
						if S <> FileIdent then
							raise EWrongIdent.Create ('Unknown file type');
						Stream.ReadBuffer (EndByte, 1);
						if EndByte <> 0 then
							raise ENoZeroByte.Create ('Unterminated string');
					except
						Abort;
					end;
					with TFileReader.Create (Stream) do try
						ReadString;
						BoardName := ReadString;
						ReadAheadIdent := ReadIdent;
						if ReadAheadIdent = 2 then
							TimeCorrection := ReadInteger;
						ReadIndent;
						TopicList.Items.BeginUpdate;
						try
							TopicList.Items.Clear;
							while True do begin
								ReadAheadIdent := ReadIdent;
								if ReadAheadIdent = -1 then begin
									Topic := TTopic.Create;
									with Topic do begin
										ListItem := TopicList.Items.Add;
										ListItem.Data := Topic;
										ReadIndent;
										RelURL := ExtractRelPath (ReadString);
										TopicName := ReadString;
										TotalReplies := ReadInteger;
										ReadFirst := ReadBoolean;
										ReadReplies := ReadInteger;
										Replies := TotalReplies;
										StartedBy := ReadString;
										LastComment := ReadString;
										LatestChange := ReadDateTime;
										UserEdited := ReadBoolean;
										MustDownload := ReadBoolean (True);
										Started := ReadDateTime;
										ReadUnindent;
									end;
								end else
									Break;
							end;
						finally
							TopicList.Items.EndUpdate;
						end;
						ReadUnindent;
						LocalCookie := ReadString;
						CanEditOwn := ReadBoolean;
						CanEditAll := ReadBoolean;
						CanDeleteOwn := ReadBoolean;
						CanDeleteAll := ReadBoolean;
						S := ReadString;
						if (not FirstStart) and (Length (S) > 0) then
							OwnID := S;
					finally
						Free;
					end;
				finally
					Stream.Free;
				end;
				with TopicList, Items do
					if Count > 0 then begin
						if TopicList.SortDirection = sdDescending then
							Selected := Item [0]
						else
							Selected := Item [Count - 1];
						Selected.Focused := True;
						Selected.MakeVisible;
					end;
			end else begin
				case ShowDefaultMessageBox ('You have never visited this forum before. Do you want to download the entire contents?', 'New Forum', mtQuestion, True) of
					idYes: RequestAll;
					idCancel: Abort;
				end;
			end;
		except
			URL := '';
			raise;
		end;
	end else
		URL := '';
end;

procedure TMainForm.SaveTopics;
var
	Stream: TFileStream;
	S: string;
	I: Integer;
begin
	if not Closed then begin
		S := URLPath;
		if Length (S) > 0 then begin
			if not FileExists (S + ListFile) then
				MkDir (S);
			Stream := TFileStream.Create (S + ListFile, fmCreate or fmShareExclusive);
			try
				Stream.WriteBuffer (PChar(FileIdent)^, Length (FileIdent) + 1);
				with TFileWriter.Create (Stream) do try
					WriteString (URL);
					WriteString (BoardName);
					WriteInteger (TimeCorrection);
					WriteIndent;
					with TopicList.Items do
						for I := 0 to Count - 1 do
							with Item [I] do
								if Assigned (Data) then
									with TTopic (Data) do begin
										WriteIndent;
										WriteString (RelURL);
										WriteString (TopicName);
										WriteInteger (Replies);
										WriteBoolean (ReadFirst);
										WriteInteger (ReadReplies);
										WriteString (StartedBy);
										WriteString (LastComment);
										WriteDateTime (LatestChange);
										WriteBoolean (UserEdited);
										WriteBoolean (MustDownload);
										WriteDateTime (Started);
										WriteUnindent;
									end;
					WriteUnindent;
					WriteString (LocalCookie);
					WriteBoolean (CanEditOwn);
					WriteBoolean (CanEditAll);
					WriteBoolean (CanDeleteOwn);
					WriteBoolean (CanDeleteAll);
					WriteString (OwnID);
				finally
					Free;
				end;
			finally
				Stream.Free;
			end;
		end;
	end;
end;

procedure TMainForm.MakeReady;
begin
	if Length (URL) <= 0 then
		Abort;
end;

procedure TMainForm.SendCommand(const HostName, Command, CommandURL, Header, Body: string);
begin
	InNewRequest := True;
	ConnectionStatus := csConnecting;
	Update;
	TextReceived := '';
	CompleteDoc := False;
	TextToSend := Command;
	if Length (CommandURL) > 0 then begin
		if UseProxy then
			TextToSend := TextToSend + ' ' + CommandURL
		else
			TextToSend := TextToSend + ' ' + ExtractRelPath (CommandURL);
	end;
	TextToSend := TextToSend + ' HTTP/1.1'#13#10'Accept: text/html'#13#10'Accept-Language: en'#13#10'User-Agent: Board Reader'#13#10'Host: ' + HostName + #13#10 + Header + 'Connection: close'#13#10;
	if UseProxy then
		TextToSend := TextToSend + 'Proxy-Connection: close'#13#10;
	TextToSend := TextToSend + #13#10 + Body;
	if Assigned (Client) then
		Client.Free;
	Client := TTCPBlockSocket.Create;
	if UseProxy then begin
		CurHost := '';
		CurIP := '';
		if (Pos ('.', Proxy) > 0) and (Proxy [1] in ['0'..'9']) then
			Client.Connect (Proxy, IntToStr (ProxyPort))
		else
			Client.Connect (Client.ResolveName (Proxy), IntToStr (ProxyPort));
	end else begin
		if (Pos ('.', HostName) > 0) and (HostName [1] in ['0'..'9']) then begin
			CurHost := '';
			CurIP := '';
			Client.Connect (HostName, '80');
		end else begin
			if CurHost <> HostName then begin
				CurHost := HostName;
				CurIP := '';
			end;
			Client.Connect (Client.ResolveName (HostName), '80');
		end;
	end;
	InNewRequest := False;
	CompleteDoc := False;
	if (Length (CurHost) > 0) and (Length (CurIP) <= 0) then begin
		CurIP := Client.GetRemoteSinIP;
		if Length (WhosOnlineIndicator.Hint) <= 0 then
			WhosOnlineIndicator.Hint := CurHost + ' = ' + CurIP;
	end;
	if CurrentAction <> raNone then begin
		if Length (TextToSend) > 0 then begin
			Client.SendString (TextToSend);
			TextToSend := '';
		end;
		ConnectionStatus := csStarting;
	end;
end;

procedure TMainForm.DocReceived;
var
	P1,
	P2: Integer;
begin
	ProcessNewCookies (TextReceived);
	P1 := Pos ('Location: ', TextReceived);
	if P1 > 0 then begin
		Delete (TextReceived, 1, P1 + Length ('Location: ') - 1);
		P1 := Pos (#13, TextReceived);
		P2 := Pos (#10, TextReceived);
		if (P1 <= 0) or ((P2 > 0) and (P2 < P1)) then
			P1 := P2;
		if P1 > 0 then
			Delete (TextReceived, P1, Length (TextReceived));
		SendCommand (ExtractHost (TextReceived), 'GET', TextReceived);
	end else
		case CurrentAction of
			raRequestAll, raRequestUpdates:
				RequestListDocReceived;
			raRequestTopic:
				RequestTopicDocReceived;
			raReloadPost:
				ReloadPostDocReceived;
			else begin
				CurrentAction := raNone;
				ConnectionStatus := csReady;
			end;
		end;
end;

procedure TMainForm.RequestUpdates(Hidden: Boolean);
var
	Cookie: string;
begin
	if Length (URL) > 0 then begin
		ShouldRequestUpdates := False;
		MakeReady;
		DownloadURL := URL;
		PageURLs := '';
		CurrentAction := raRequestUpdates;
		if Hidden then
			Cookie := ''
		else
			Cookie := GetCookieHeader;
		SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL, Cookie);
	end;
end;

procedure TMainForm.RequestAll;
begin
	if Length (URL) > 0 then begin
		ShouldRequestUpdates := False;
		MakeReady;
		DownloadURL := URL;
		PageURLs := '';
		CurrentAction := raRequestAll;
		SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL);
	end;
end;

procedure TMainForm.RequestTopicListPage;
begin
	RequestUpdates (True);
end;

procedure TMainForm.RequestListDocReceived;
var
	Doc,
	Heading,
	TopicURL,
	S,
	TempS: string;
	I,
	P: Integer;
	Headings,
	Cells: TStringList;
	Item: TListItem;
	Topic: TTopic;
	HadNewItems,
	DeleteParenthesis,
	RequestedAll: Boolean;
	NewReplies: Integer;
	NewLatestChange: TDateTime;
	FirstPage,
	IsNewTopic: Boolean;
	MarkReadURL: string;
	MemberCount,
	GuestCount,
	AnonymousCount: Integer;
	MembersOnline: string;
begin
	HadNewItems := False;
	MarkReadURL := '';
	try
		try
			Doc := TextReceived;
			FirstPage := Length (PageURLs) <= 0;
			if DeleteToAfterFirst ('<title>', Doc, S) then
				if DeleteToAfterFirst ('</title>', Doc, S, 2000) and (Length (BoardName) <= 0) and (DownloadURL = URL) then
					BoardName := DeHTMLize (S);
			P := Pos ('Logged in as', Doc);
			if P > 0 then begin
				Inc (P, Length ('Logged in as'));
				OwnID := Copy (Doc, P, 100);
				P := Pos ('<', OwnID);
				if P > 0 then begin
					Delete (OwnID, P, Length (OwnID));
					TrimWhiteSpace (OwnID);
				end else
					OwnID := '';
			end;
			P := Pos ('in the last', Doc);
			if P > 0 then begin
				S := Copy (Doc, P, 1000);
				MemberCount := 0;
				GuestCount := 0;
				AnonymousCount := 0;
				P := Pos ('member', LowerCase (S)) - 1;
				if P > 0 then begin
					while (P > 0) and (S [P] in [' ', #13, #10, #9]) do
						Dec (P);
					while (P > 0) and (S [P] in ['0'..'9']) do begin
						MemberCount := MemberCount * 10 + Ord (S [P]) - Ord ('0');
						Dec (P);
					end;
				end;
				P := Pos ('guest', LowerCase (S)) - 1;
				if P > 0 then begin
					while (P > 0) and (S [P] in [' ', #13, #10, #9]) do
						Dec (P);
					while (P > 0) and (S [P] in ['0'..'9']) do begin
						GuestCount := GuestCount * 10 + Ord (S [P]) - Ord ('0');
						Dec (P);
					end;
				end;
				P := Pos ('anonymous', LowerCase (S)) - 1;
				if P > 0 then begin
					while (P > 0) and (S [P] in [' ', #13, #10, #9]) do
						Dec (P);
					while (P > 0) and (S [P] in ['0'..'9']) do begin
						AnonymousCount := AnonymousCount * 10 + Ord (S [P]) - Ord ('0');
						Dec (P);
					end;
				end;
				MembersOnline := '';
				DeleteToAfterFirst ('', S, TempS);
				while (DeleteToAfterFirst ('<a ', S, TempS)) do begin
					DeleteToAfterFirst ('>', S, TempS);
					DeleteToAfterFirst ('</a', S, TempS);
					P := Pos ('<', TempS);
					while P > 0 do begin
						I := Pos ('>', Copy (TempS, P, Length (TempS)));
						if I > 0 then
							Delete (TempS, P, I);
						P := Pos ('<', TempS);
					end;
					TrimHTML (TempS);
					if Length (TempS) > 0 then begin
						TempS := DeHTMLize (TempS);
						if Length (MembersOnline) > 0 then
							MembersOnline := MembersOnline + ', ';
						MembersOnline := MembersOnline + GetAlias (TempS);
					end;
					P := Pos ('<', S);
					while (P > 0) and (LowerCase (Copy (S, P, Length ('<a '))) <> '<a ') do begin
						if LowerCase (Copy (S, P, Length ('</tr>'))) = '</tr>' then begin
							S := '';
							Break;
						end;
						Delete (S, 1, P);
						P := Pos ('<', S);
					end;
				end;
				TempS := '';
				if MemberCount > 0 then begin
					if MemberCount = 1 then
						TempS := '1 Member'
					else
						TempS := IntToStr (MemberCount) + ' Members';
					if Length (MembersOnline) > 0 then
						TempS := TempS + ': ' + MembersOnline;
				end;
				if GuestCount > 0 then begin
					if Length (TempS) > 0 then
						TempS := TempS + #13#10;
					if GuestCount = 1 then
						TempS := TempS + '1 Guest'
					else
						TempS := TempS + IntToStr (GuestCount) + ' Guests';
				end;
				if AnonymousCount > 0 then begin
					if Length (TempS) > 0 then
						TempS := TempS + #13#10;
					TempS := TempS + IntToStr (AnonymousCount) + ' Anonymous';
				end;
				if Length (TempS) > 0 then
					TempS := TimeToStr (Time) + ':'#13#10 + TempS;
				if (Length (CurHost) > 0) and (Length (CurIP) > 0) then
					TempS := TempS + #13#10#13#10 + CurHost + ' = ' + CurIP;
				WhosOnlineIndicator.Hint := TempS;
			end;
			if (CurrentAction = raRequestUpdates) and MarkForumsRead then begin
				if DeleteToFirst ('class=headings>', Doc, S) and DeleteToAfterFirst ('Mark Forum Read', S, Doc) then begin
					TrimWhiteSpace (Doc);
					if (Length (Doc) > 0) and (Doc [Length (Doc)] = '>') then begin
						Delete (Doc, Length (Doc), 1);
						P := LastPos (' ', Doc);
						if P > 0 then begin
							Delete (Doc, 1, P);
							if DeleteToAfterFirst ('href=', Doc, S, 0) and (Length (Doc) > 0) then begin
								if Doc [1] = '"' then
									Delete (Doc, 1, 1);
								if Doc [Length (Doc)] = '"' then
									Delete (Doc, Length (Doc), 1);
								MarkReadURL := DeHTMLize (Doc);
							end;
						end;
					end;
				end;
				Doc := TextReceived;
			end;
			if (CurrentAction = raRequestAll) and FirstPage then begin
				S := Doc;
				if DeleteToFirst ('class=headings>', S, Doc) and DeleteToAfterFirst ('page ', Doc, S) then begin
					while DeleteToAfterFirst ('page ', Doc, S) do;
					if DeleteToAfterFirst ('</td>', Doc, S) then
						Doc := S;
					if not DeleteToAfterFirst ('<table', Doc, S) then
						S := Doc;
					if Pos ('<td', LowerCase (S)) <= 0 then begin
						while DeleteToAfterFirst ('<a ', S, TempS) do begin
							if DeleteToAfterFirst ('href=', S, TempS) then begin
								if DeleteToAfterFirst ('>', S, TempS) then begin
									if Length (TempS) > 0 then begin
										if TempS [1] = '"' then begin
											Delete (TempS, 1, 1);
											Delete (TempS, Pos ('"', TempS), Length (TempS));
										end else begin
											if Pos (' ', TempS) > 0 then
												Delete (TempS, Pos (' ', TempS), Length (TempS));
										end;
										PageURLs := PageURLs + DeHTMLize (TempS) + ';';
									end;
								end;
							end;
						end;
					end;
				end;
				Doc := TextReceived;
			end else
				Delete (PageURLs, 1, Pos (';', PageURLs));
			Headings := TStringList.Create;
			Cells := TStringList.Create;
			try
				while DeleteToAfterFirst ('class=headings>', Doc, S) do begin
					DeleteToAfterFirst ('</td>', Doc, S, 400);
					TrimHTML (S);
					Headings.Add (S);
				end;
				DeleteWhiteSpace (Doc);
				if not DeleteToAfterFirst ('</tr>', Doc, S, 0) then
					Abort;
				DeleteWhiteSpace (Doc);
				while DeleteToAfterFirst ('<tr', Doc, S, 0) do begin
					DeleteToAfterFirst ('>', Doc, S);
					TopicURL := '';
					Cells.Clear;
					for I := 0 to Headings.Count - 1 do begin
						DeleteWhiteSpace (Doc);
						if not DeleteToAfterFirst ('<td', Doc, S, 0) then
							Abort;
						DeleteToAfterFirst ('>', Doc, S);
						if not DeleteToAfterFirst ('</td>', Doc, S, 5000) then
							Abort;
						DeleteParenthesis := False;
						if UpperCase (Headings.Strings [I]) = 'TOPIC' then begin
							DeleteWhiteSpace (S);
							if DeleteToAfterFirst ('<a', S, TempS, 0) and DeleteToAfterFirst ('href=', S, TempS) then begin
								if DeleteToAfterFirst ('"', S, TempS, 0) then begin
									if not DeleteToAfterFirst ('"', S, TopicURL, 1000) then
										Abort;
								end else begin
									if not DeleteToFirst ('>', S, TopicURL, 1000) then
										Abort;
									if Pos (' ', TopicURL) > 0 then
										Delete (TopicURL, Pos (' ', TopicURL), Length (TopicURL));
								end;
								TopicURL := DeHTMLize (TopicURL);
								if not DeleteToAfterFirst ('>', S, TempS, 1000) then
									Abort;
							end;
							if Pos ('<a', LowerCase (S)) > 0 then begin
								Delete (S, Pos ('<a', LowerCase (S)), Length (S));
								DeleteParenthesis := True;
							end;
						end;
						TrimHTML (S);
						if DeleteParenthesis and (Length (S) > 0) and (S [Length (S)] = '(') then begin
							Delete (S, Length (S), 1);
							TrimWhiteSpace (S);
						end;
						Cells.Add (S);
					end;
					DeleteWhiteSpace (Doc);
					if not DeleteToAfterFirst ('</tr>', Doc, S, 0) then
						Abort;
					DeleteWhiteSpace (Doc);
					if (Cells.Count = Headings.Count) and (Length (TopicURL) > 0) then begin
						if not HadNewItems then begin
							HadNewItems := True;
							if InstantUpdates then
								TopicList.Items.BeginUpdate;
						end;
						IsNewTopic := False;
						if Length (TopicURL) > 0 then
							Item := FindTopicByURL (TopicURL)
						else
							Item := nil;
						if Assigned (Item) and Assigned (Item.Data) then
							Topic := TTopic (Item.Data)
						else begin
							Item := TopicList.Items.Add;
							Topic := TTopic.Create;
							Topic.ListItem := Item;
							Item.Data := Topic;
							Topic.RelURL := ExtractRelPath (TopicURL);
							Topic.MustDownload := True;
							IsNewTopic := True;
						end;
						with Topic do begin
							for I := 0 to Headings.Count - 1 do begin
								S := Cells.Strings [I];
								Heading := UpperCase (Headings.Strings [I]);
								if (Heading = 'TOPIC') and (not UserEdited) then
									TopicName := S
								else if Heading = 'REPLIES' then try
									NewReplies := StrToInt (S);
									if Replies <> NewReplies then begin
										Replies := NewReplies;
										MustDownload := True;
									end;
								except end else if Heading = 'LAST COMMENT' then begin
									P := Pos (#13, S);
									if P > 0 then begin
										try
											ShortDateFormat := 'mm/dd/yy';
											DateSeparator := '/';
											NewLatestChange := StrToDateTime (Trim (Copy (S, 1, P - 1)));
											if LatestChange <> NewLatestChange then begin
												LatestChange := NewLatestChange;
												MustDownload := True;
											end;
										except end;
										S := Trim (Copy (S, P + 1, Length (S)));
										if LowerCase (Copy (S, 1, Length ('by:'))) = 'by:' then
											S := Trim (Copy (S, Length ('by:') + 1, Length (S)));
										LastComment := S;
									end else try
										ShortDateFormat := 'mm/dd/yy';
										DateSeparator := '/';
										LatestChange := StrToDateTime (S);
									except end;
								end else if Heading = 'STARTED BY' then
									StartedBy := S;
							end;
						end;
						if IsNewTopic and (CurrentAction <> raRequestAll) then
							Item.MakeVisible;
					end;
				end;
			finally
				Cells.Free;
				Headings.Free;
			end;
		except end;
	finally
		if HadNewItems and InstantUpdates then
			TopicList.Items.EndUpdate;
		if HadNewItems then
			SaveTopics;
		if MarkForumsRead and HadNewItems and (CurrentAction = raRequestUpdates) and (Length (MarkReadURL) > 0) then begin
			DownloadURL := MarkReadURL;
			PageURLs := '';
			CurrentAction := raMarkForumRead;
			SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL, GetCookieHeader);
		end else if HadNewItems and (CurrentAction = raRequestAll) and (Pos (';', PageURLs) > 0) then
			SendCommand (ExtractHost (DownloadURL), 'GET', Copy (PageURLs, 1, Pos (';', PageURLs) - 1))
		else begin
			RequestedAll := CurrentAction = raRequestAll;
			CurrentAction := raNone;
			ConnectionStatus := csReady;
			if RequestedAll and (ShowDefaultMessageBox ('The list has been downloaded successfully.  Do you want to append the contents of another forum?', 'Finished', mtQuestion) = idYes) then begin
				with TAppendForumForm.Create (Self) do try
					OrigURL := URL;
					URLEdit.Text := URL;
					if ShowModal = mrOK then begin
						DownloadURL := URLEdit.Text;
						PageURLs := '';
						CurrentAction := raRequestAll;
						SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL);
					end;
				finally
					Free;
				end;
			end else
				WaitBeforeDownload := 1;
		end;
	end;
end;

procedure TMainForm.TopicListDeletion(Sender: TObject; Item: TListItem);
begin
	with Item do
		if Assigned (Data) then
			with TTopic (Data) do begin
				ListItem := nil;
				Free;
			end;
end;

procedure TMainForm.AddSubItem(ListItem: TListItem; Column: TListColumn; const S: string);
var
	Idx: Integer;
begin
	if Assigned (ListItem) and Assigned (Column) and (Length (S) > 0) then begin
		if Column = MainColumn then
			ListItem.Caption := S
		else begin
			Idx := Column.Index;
			if Idx > MainColumn.Index then
				Dec (Idx);
			while ListItem.SubItems.Count < Idx do
				ListItem.SubItems.Add ('');
			if Idx = ListItem.SubItems.Count then
				ListItem.SubItems.Add (S)
			else
				ListItem.SubItems.Strings [Idx] := S
		end;
	end;
end;

procedure TMainForm.TopicListDblClick(Sender: TObject);
begin
	with TopicList do
		if Assigned (Selected) then
			with Selected do
				if Assigned (Data) then
					with TTopic (Data) do
						Show;
end;

procedure TMainForm.SetBoardName(const Value: string);
begin
	if FBoardName <> Value then begin
		FBoardName := Value;
		if Length (Value) > 0 then
			Caption := 'Board Reader - ' + Value
		else
			Caption := 'Board Reader';
		Application.Title := Caption;
	end;
	if (Length (URL) > 0) and (Length (Value) > 0) then
		with TMemIniFile.Create (HomeDir + PrefsFile) do try
			if (not ValueExists ('Favorites', URL)) or (ReadString ('Favorites', URL, '') <> Value) then begin
				WriteString ('Favorites', URL, Value);
				UpdateFile;
				LoadFavorites;
			end;
		finally
			Free;
		end;
end;

procedure TMainForm.SetConnectionStatus(const Value: TConnectionStatus);
begin
	FConnectionStatus := Value;
	case Value of
		csReady:
			StatusText := 'Ready';
		csConnecting:
			StatusText := 'Connecting...';
		csStarting:
			case CurrentAction of
				raGlobalLogin, raLocalLogin:
					StatusText := 'Logging in...';
				raRequestUpdates, raRequestAll:
					StatusText := 'Starting List Download...';
				raMarkForumRead:
					StatusText := 'Marking Forum as Read...';
				raRequestTopic:
					StatusText := 'Starting Topic Download...';
				raPostTopic:
					StatusText := 'Posting Topic...';
				raPostReply:
					StatusText := 'Posting Reply...';
				raEditPost:
					StatusText := 'Editing Post...';
				raDeleteTopic:
					StatusText := 'Deleting Topic...';
				raDeletePost:
					StatusText := 'Deleting Post...';
				raReloadPost:
					StatusText := 'Starting Post Download...';
				else
					StatusText := 'Starting Download...';
			end;
		csDownloading:
			case CurrentAction of
				raGlobalLogin, raLocalLogin:
					StatusText := 'Downloading Cookie...';
				raRequestUpdates, raRequestAll:
					StatusText := 'Downloading List...';
				raMarkForumRead:
					CancelAction;
				raRequestTopic:
					if Assigned (DownloadTopic) then
						StatusText := Format ('Downloading Topic ''%s''...', [DownloadTopic.TopicName]);
				raPostTopic, raDeleteTopic: begin
					CancelAction;
					ExecProc (RequestTopicListPage);
				end;
				raPostReply, raEditPost, raDeletePost: begin
					if Assigned (DownloadTopic) then begin
						DownloadTopic.MustDownload := True;
						WaitBeforeDownload := 1;
						NextDownloadsNotAnonymous := True;
					end;
					CancelAction;
				end;
				raReloadPost:
					StatusText := 'Downloading Post...';
				else
					StatusText := 'Downloading...';
			end;
	end;
end;

procedure TMainForm.SearchButtonClick(Sender: TObject);
begin
	with TSearchForm.Create (Self) do try
		SearchStringEdit.Text := StringReplace (SearchWords.CommaText, ',', ', ', [rfReplaceAll]);
		ShowModal;
	finally
		Free;
	end;
end;

procedure TMainForm.CancelAction;
begin
	if Assigned (Client) then begin
		Client.Free;
		Client := nil;
	end;
	CurrentAction := raNone;
	ConnectionStatus := csReady;
	DownloadTopic := nil;
	TextReceived := '';
	CompleteDoc := False;
end;

function TMainForm.FindTopicByURL(AURL: string): TListItem;
function TestItem(Item: TListItem): Boolean;
begin
	Result := False;
	with Item do
		if Assigned (Data) then
			with TTopic (Data) do
				Result := (Length (RelURL) > 0) and (LowerCase (RelURL) = Copy (AURL, 1, Length (RelURL)));
end;
var
	I: Integer;
begin
	Result := nil;
	AURL := LowerCase (ExtractRelPath (AURL));
	with TopicList.Items do
		if TopicList.SortDirection = sdDescending then begin
			for I := 0 to Count - 1 do
				if TestItem (Item [I]) then begin
					Result := Item [I];
					Break;
				end;
		end else begin
			for I := Count - 1 downto 0 do
				if TestItem (Item [I]) then begin
					Result := Item [I];
					Break;
				end;
		end;
end;

procedure TMainForm.TopicListSelectItem(Sender: TObject; Item: TListItem;
	Selected: Boolean);
begin
	if not UpdatingPopup then begin
		UpdatingPopup := True;
		ExecProc (UpdatePopupMenu);
	end;
end;

procedure TMainForm.MarkAsReadMenuItemClick(Sender: TObject);
var
	CurItem: TListItem;
begin
	CurItem := TopicList.Selected;
	while Assigned (CurItem) do begin
		with CurItem do
			if Assigned (Data) then
				with TTopic (Data) do
					ReadAll := True;
		CurItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
	end;
	SaveTopics;
end;

procedure TMainForm.MarkAsUnreadMenuItemClick(Sender: TObject);
var
	CurItem: TListItem;
begin
	CurItem := TopicList.Selected;
	while Assigned (CurItem) do begin
		with CurItem do
			if Assigned (Data) then
				with TTopic (Data) do
					ReadAll := False;
		CurItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
	end;
	SaveTopics;
end;

procedure TMainForm.TopicListEdited(Sender: TObject; Item: TListItem;
	var S: WideString);
begin
	with Item do
		if Assigned (Data) then
			with TTopic (Data) do begin
				TopicName := S;
				UserEdited := True;
			end;
	SaveTopics;
	TopicList.OnKeyDown := TopicListKeyDown;
end;

procedure TMainForm.SetURL(Value: string);
var
	Idx: Integer;
begin
	CancelAction;
	with URLEdit do begin
		if NewPostCount > 0 then begin
			if ShowDefaultMessageBox ('This forum contains unverified new posts. Are you sure you want to switch forums?', 'Warning', mtQuestion) = idNo then begin
				Text := URL;
				Abort;
			end;
		end;
		if ItemIndex >= 0 then
			Idx := ItemIndex
		else
			Idx := Items.IndexOf (Value);
		if (Idx >= 0) and (UpperCase (Items.Strings [Idx]) = UpperCase (Value)) then begin
			if Assigned (Items.Objects [Idx]) then begin
				with TStringObject (Items.Objects [Idx]) do
					Value := Str;
				Text := Value;
				SelectAll;
			end;
		end;
	end;
	if FURL <> Value then try
		FURL := Value;
		BoardName := '';
		with TopicList.Items do begin
			BeginUpdate;
			try
				Clear;
			finally
				EndUpdate;
			end;
		end;
		WhosOnlineIndicator.Hint := '';
		if Length (Value) > 0 then begin
			LoadTopics;
			if CurrentAction <> raRequestAll then begin
				HideNextUpdate := HideInitUpdates;
				ShouldRequestUpdates := True;
			end;
		end;
	except
		FURL := '';
		raise;
	end;
	RefreshMenuItem.Enabled := Length (URL) > 0;
	RefreshAllMenuItem.Enabled := Length (URL) > 0;
	SearchButton.Enabled := Length (URL) > 0;
	NewTopicButton.Enabled := Length (URL) > 0;
end;

procedure TMainForm.URLEditKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	if Key = 4100 then
		URL := URLEdit.Text;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	case Key of
		4096:
			if CurrentAction = raNone then
				Close
			else
				CancelAction;
	end;
end;

procedure TMainForm.UpdatePopupMenu;
var
	CurItem: TListItem;
	AllRead,
	AllUnread,
	AllDownloaded: Boolean;
begin
	AllRead := True;
	AllUnread := True;
	AllDownloaded := True;
	CurItem := TopicList.Selected;
	while Assigned (CurItem) and (AllRead or AllUnread) do begin
		with CurItem do
			if Assigned (Data) then begin
				with TTopic (Data) do begin
					if ReadAll then
						AllUnread := False
					else
						AllRead := False;
					if MustDownload then
						AllDownloaded := False;
				end;
			end;
		CurItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
	end;
	MarkAsReadMenuItem.Enabled := not AllRead;
	MarkAsUnreadMenuItem.Enabled := not AllUnread;
	RedownloadMenuItem.Enabled := Assigned (TopicList.Selected);
	StopDownloadingMenuItem.Enabled := not AllDownloaded;
	UpdatingPopup := False;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	try
		Closing := True;
		CancelAction;
		SaveSettings;
		Closed := True;
		with TopicList.Items do
			if Count > 0 then begin
				BeginUpdate;
				try
					Clear;
				finally
					EndUpdate;
				end;
			end;
	except end;
end;

procedure TMainForm.TopicDownloadTimerTick(Sender: TObject);
var
	S: string;
begin
	if Assigned (Client) and Client.CanRead (0) then begin
		S := Client.RecvPacket (512);
		if Length (S) <= 0 then begin
			Client.Free;
			Client := nil;
			if (not InNewRequest) and (not CompleteDoc) then begin
				CompleteDoc := True;
				ExecProc(DocReceived);
			end;
		end else begin
			if CurrentAction <> raNone then begin
				ConnectionStatus := csDownloading;
				if (Length (TextToSend) <= 0) and (ConnectionStatus = csDownloading) and (Pos (S, TextReceived) <= 0) then begin
					TextReceived := TextReceived + S;
					CompleteDoc := Pos ('</HTML>', UpperCase (TextReceived)) > 0;
					if CompleteDoc then
						ExecProc(DocReceived);
				end;
			end;
		end;
	end;
	if ShouldSetURL then begin
		ShouldSetURL := False;
		URL := URLEdit.Text;
	end else begin
		if (CurrentAction = raNone) and (InErrorCount <= 0) then begin
			if WaitBeforeDownload > 0 then
				Dec (WaitBeforeDownload)
			else begin
				if ShouldRequestUpdates then begin
					RequestUpdates (HideNextUpdate);
					HideNextUpdate := HideAutoUpdates;
				end else if TopicsToDownload > 0 then
					RequestNextTopic (HideTopicDownloads and (not NextDownloadsNotAnonymous))
				else
					NextDownloadsNotAnonymous := False;
			end;
		end;
	end;
end;

procedure TMainForm.TopicListKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	case Key of
		4100:
			TopicListDblClick (Sender);
		4103:
			DeleteSelectedTopics;
		4148:
			RequestUpdates;
	end;
end;

procedure TMainForm.RequestNextTopic(Hidden: Boolean);
function TestItem(Item: TListItem): Boolean;
begin
	Result := False;
	with Item do
		if Assigned (Data) then
			with TTopic (Data) do begin
				Result := MustDownload;
				if Result then
					RequestTopic (TTopic (Data), Hidden);
			end;
end;
var
	I: Integer;
begin
	if Length (URL) <= 0 then
		Abort;
	with TopicList.Items do
		if TopicList.SortDirection = sdDescending then begin
			for I := 0 to Count - 1 do
				if TestItem (Item [I]) then
					Break;
		end else begin
			for I := Count - 1 downto 0 do
				if TestItem (Item [I]) then
					Break;
		end;
end;

procedure TMainForm.RequestTopic(Topic: TTopic; Hidden: Boolean);
var
	Cookie: string;
begin
	if Length (URL) <= 0 then
		Abort;
	DownloadTopic := Topic;
	DownloadURL := Topic.AbsURL;
	PageURLs := '';
	if Length (DownloadURL) > 0 then begin
		CurrentAction := raRequestTopic;
		if Hidden then
			Cookie := ''
		else
			Cookie := GetCookieHeader;
		SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL, Cookie);
	end;
end;

procedure TMainForm.RequestTopicDocReceived;
var
	P,
	P2,
	NewIndex: Integer;
	Doc,
	S,
	TempS,
	TempStr,
	NewUserID,
	NewTitle,
	NewText,
	NewEditorID: string;
	NewPostTime,
	NewEditTime: TDateTime;
	NewExtras: TPostExtras;
	FirstTitle,
	XLastComment: string;
	XLatestChange: TDateTime;
	Links: TStringList;
	FirstPage,
	StupidTable: Boolean;
begin
	S := TextReceived;
	FirstPage := Length (PageURLs) <= 0;
	FirstTitle := '';
	XLastComment := '';
	XLatestChange := 0;
	try
		try
			if not DeleteToAfterFirst ('class=headings>', S, Doc) then
				Abort;
			if FirstPage then begin
				if DeleteToAfterFirst ('page ', Doc, S) then begin
					if not DeleteToAfterFirst ('</td>', Doc, S) then
						S := Doc;
					if Pos ('<td', LowerCase (S)) <= 0 then begin
						while DeleteToAfterFirst ('<a ', S, TempS) do begin
							if DeleteToAfterFirst ('href=', S, TempS) then begin
								if DeleteToAfterFirst ('>', S, TempS) then begin
									if Length (TempS) > 0 then begin
										if TempS [1] = '"' then begin
											Delete (TempS, 1, 1);
											Delete (TempS, Pos ('"', TempS), Length (TempS));
										end else begin
											if Pos (' ', TempS) > 0 then
												Delete (TempS, Pos (' ', TempS), Length (TempS));
										end;
										PageURLs := PageURLs + DeHTMLize (TempS) + ';';
									end;
								end;
							end;
						end;
					end;
				end;
			end else
				Delete (PageURLs, 1, Pos (';', PageURLs));
			Doc := TextReceived;
			if not DeleteToAfterFirst ('class=headings>', Doc, S) then
				Abort;
			if not DeleteToAfterFirst ('</tr>', Doc, S) then
				Abort;
			DeleteWhiteSpace (Doc);
			if Assigned (DownloadTopic) then
				with DownloadTopic do begin
					if Length (FileName) > 0 then begin
						if FirstPage then
							TopicContents.Clear;
						NewIndex := -1;
						while DeleteToAfterFirst ('<tr', Doc, S, 0) do begin
							StupidTable := False;
							DeleteToAfterFirst ('>', Doc, S);
							DeleteWhiteSpace (Doc);
							if not DeleteToAfterFirst ('<td', Doc, S, 0) then begin
								DeleteToAfterFirst ('</tr>', Doc, S);
								DeleteWhiteSpace (Doc);
								Continue;
							end;
							DeleteToAfterFirst ('>', Doc, S);
							DeleteWhiteSpace (Doc);
							if DeleteToAfterFirst ('<table', Doc, S, 0) then begin
								StupidTable := True;
								DeleteToAfterFirst ('<td', Doc, S);
								DeleteToAfterFirst ('<td', Doc, S);
								DeleteToAfterFirst ('>', Doc, S);
							end;
							if not DeleteToAfterFirst ('</td>', Doc, S, 5000) then begin
								DeleteToAfterFirst ('</tr>', Doc, S);
								DeleteWhiteSpace (Doc);
								Continue;
							end;
							NewUserID := '';
							if DeleteToAfterFirst ('<span class=title>', S, TempS) then begin
								if DeleteToAfterFirst ('</span>', S, NewUserID) then begin
									TrimHTML (NewUserID);
									XLastComment := NewUserID;
								end;
							end;
							Inc (NewIndex);
							P := Pos ('index=', S);
							if P > 0 then begin
								Inc (P, Length ('index='));
								TempS := '';
								while (P <= Length (S)) and (S [P] in ['0'..'9']) do begin
									TempS := TempS + S [P];
									Inc (P);
								end;
								try
									NewIndex := StrToInt (TempS);
								except end;
							end;
							if not HideTopicDownloads then begin
								if Pos ('edit', LowerCase (S)) > 0 then begin
									CanEditOwn := True;
									if (Length (OwnID) > 0) and (NewUserID <> OwnID) then
										CanEditAll := True;
								end else begin
									CanEditAll := False;
									if (Length (OwnID) > 0) and (NewUserID = OwnID) then
										CanEditOwn := False;
								end;
								if Pos ('del', LowerCase (S)) > 0 then begin
									CanDeleteOwn := True;
									if (Length (OwnID) > 0) and (NewUserID <> OwnID) then
										CanDeleteAll := True;
								end else begin
									CanDeleteAll := False;
									if (Length (OwnID) > 0) and (NewUserID = OwnID) then
										CanDeleteOwn := False;
								end;
							end;
							NewPostTime := ProgStartTime;
							if DeleteToAfterFirstAndAdd (' am)', S, TempS) or DeleteToAfterFirstAndAdd (' pm)', S, TempS) then begin
								while Pos ('(', TempS) > 0 do
									Delete (TempS, 1, Pos ('(', TempS));
								if Pos (')', TempS) > 0 then
									Delete (TempS, Pos (')', TempS), Length (TempS));
								try
									ShortDateFormat := 'mm/dd/yy';
									DateSeparator := '/';
									NewPostTime := StrToDateTime (TempS);
									if DownloadTopic.Started > NewPostTime then
										DownloadTopic.Started := NewPostTime;
									if XLatestChange < NewPostTime then
										XLatestChange := NewPostTime;
								except end;
							end;
							DeleteWhiteSpace (Doc);
							if DeleteToAfterFirst ('<td', Doc, S, 0) then begin
								DeleteToAfterFirst ('>', Doc, S);
								if not DeleteToAfterFirst ('</td>', Doc, S) then
									Abort;
								while Pos ('<table', LowerCase (S)) > 0 do begin
									if Pos ('</table>', LowerCase (S)) > 0 then begin
										if not (DeleteToAfterFirst ('<table', S, TempStr)
											and DeleteToAfterFirst ('>', S, TempS)
											and DeleteToAfterFirst ('</table>', S, TempS)) then
											Abort;
										if Pos ('<table', LowerCase (S)) > 0 then
											Abort;
										S := TempStr + TempS + S;
									end else begin
										if not DeleteToAfterFirst ('</td>', Doc, TempS) then
											Abort;
										S := S + TempS;
									end;
								end;
								NewTitle := '';
								if DeleteToAfterFirst ('<span class=title>', S, TempS) then begin
									if DeleteToAfterFirst ('</span>', S, NewTitle) then begin
										TrimHTML (NewTitle);
										if Length (FirstTitle) <= 0 then
											FirstTitle := NewTitle;
									end;
								end;
								NewText := S;
								Links := TStringList.Create;
								try
									while DeleteToAfterFirst ('<a ', S, TempS) do begin
										if DeleteToAfterFirst ('href=', S, TempS) then begin
											if DeleteToAfterFirst ('>', S, TempS) then begin
												if Length (TempS) > 0 then begin
													if TempS [1] = '"' then begin
														Delete (TempS, 1, 1);
														Delete (TempS, Pos ('"', TempS), Length (TempS));
													end else begin
														if Pos (' ', TempS) > 0 then
															Delete (TempS, Pos (' ', TempS), Length (TempS));
													end;
													if Pos ('publicprofile', LowerCase (TempS)) <= 0 then
														Links.Add (DeHTMLize (TempS));
												end;
											end;
										end;
									end;
									NewExtras := [];
									repeat
										P := Pos ('<!--EZCODE EMOTICON START', UpperCase (NewText));
										if P > 0 then begin
											Delete (NewText, P, Length ('<!--EZCODE EMOTICON START'));
											P2 := Pos ('-->', Copy (NewText, P, Length (NewText)));
											if P2 > 0 then
												Delete (NewText, P + P2 - 1, Length ('-->'));
											Include (NewExtras, peEmoticons);
										end;
									until P <= 0;
									NewText := StringReplace (NewText, '<!--EZCODE EMOTICON END-->', '', [rfReplaceAll, rfIgnoreCase]);
									if Pos ('<!--EZCODE AUTOLINK START-->', UpperCase (NewText)) > 0 then
										Include (NewExtras, peAutoLinks);
									NewText := StringReplace (NewText, '<!--EZCODE AUTOLINK START-->', '', [rfReplaceAll, rfIgnoreCase]);
									NewText := StringReplace (NewText, '<!--EZCODE AUTOLINK END-->', '', [rfReplaceAll, rfIgnoreCase]);
									if Pos ('<!--EZCODE', UpperCase (NewText)) > 0 then
										Include (NewExtras, peEZCodes);
									TrimHTML (NewText);
									NewEditorID := '';
									NewEditTime := ProgStartTime;
									if (LowerCase (Copy (NewText, Length (NewText) - Length (' am') + 1, Length (' am'))) = ' am') or (LowerCase (Copy (NewText, Length (NewText) - Length (' pm') + 1, Length (' pm'))) = ' pm') and (Pos ('edited by', LowerCase (NewText)) > 0) then begin
										P := LastPos ('edited by', LowerCase (NewText));
										if P > 0 then begin
											NewEditorID := Copy (NewText, P + Length ('edited by'), Length (NewText));
											Delete (NewText, P, Length (NewText));
											TrimWhiteSpace (NewText);
											DeleteWhiteSpace (NewEditorID);
											if (Length (NewEditorID) > 0) and (NewEditorID [1] = ':') then begin
												Delete (NewEditorID, 1, 1);
												DeleteWhiteSpace (NewEditorID);
											end;
											P := Pos (' at', LowerCase (NewEditorID));
											if P > 0 then begin
												TempS := Copy (NewEditorID, P + Length (' at'), Length (NewEditorID));
												if Length (TempS) > 0 then begin
													if TempS [1] = ':' then begin
														Delete (TempS, 1, 1);
														DeleteWhiteSpace (TempS);
													end;
													Delete (NewEditorID, P, Length (NewEditorID));
													TrimWhiteSpace (NewEditorID);
													try
														ShortDateFormat := 'mm/dd/yy';
														DateSeparator := '/';
														NewEditTime := StrToDateTime (Trim (TempS));
														if XLatestChange < NewEditTime then
															XLatestChange := NewEditTime;
													except
														NewEditorID := '';
													end;
												end else
													NewEditorID := '';
											end else
												NewEditorID := '';
										end;
									end;
									if StupidTable then
										DeleteToAfterFirst ('</table>', Doc, S);
									DeleteToAfterFirst ('</tr>', Doc, S);
									DeleteWhiteSpace (Doc);
									with TTopicPost (TopicContents.Add) do begin
										PostIndex := NewIndex;
										UserID := NewUserID;
										Title := NewTitle;
										Text := NewText;
										PostTime := NewPostTime;
										if Length (NewEditorID) > 0 then begin
											EditorID := NewEditorID;
											EditTime := NewEditTime;
										end;
										LinkURLs.AddStrings (Links);
										PostExtras := NewExtras;
									end;
								finally
									Links.Free;
								end;
							end else begin
								DeleteToAfterFirst ('</tr>', Doc, S);
								DeleteWhiteSpace (Doc);
							end;
						end;
						TTopicContents(TopicContents).Save;
						if XLatestChange > LatestChange then
							LatestChange := XLatestChange
						else
							XLatestChange := 0;
						if Replies < TopicContents.Count - 1 then begin
							Replies := TopicContents.Count - 1;
							LastComment := XLastComment;
							XLatestChange := LatestChange;
						end;
						if XLatestChange <> 0 then
							SaveTopics;
					end;
				end;
		except end;
	finally
		if Pos (';', PageURLs) > 0 then begin
			if HideTopicDownloads then
				S := ''
			else
				S := GetCookieHeader;
			SendCommand (ExtractHost (DownloadURL), 'GET', Copy (PageURLs, 1, Pos (';', PageURLs) - 1), S)
		end else begin
			CurrentAction := raNone;
			ConnectionStatus := csReady;
			if Assigned (DownloadTopic) then
				with DownloadTopic do begin
					MustDownload := False;
					SaveTopics;
					if (not Assigned (TopicForm)) and Assigned (NewTopicForm) then
						with TTopicForm (NewTopicForm) do
							if EditingPost and Assigned (EditedPost) then
								if UpperCase (EditedPost.Title.Text) = UpperCase (FirstTitle) then begin
									DownloadTopic.TopicForm := NewTopicForm;
									Topic := DownloadTopic;
								end;
					with TTopicContents (TopicContents) do begin
						if Length (NewPost) > 0 then begin
							if Count > 0 then begin
								if Length (OwnID) > 0 then
									with TTopicPost (Items [Count - 1]) do begin
										if UserID = OwnID then begin
											if UpperCase (Text) <> 'HTML COMMENTS ARE NOT ALLOWED' then begin
												if Abs (Length (NewPost) - Length (Text)) <= Length (NewPost) div 10 then begin
													NewPost := '';
													ReadAll := True;
													SaveTopics;
												end else
													ShowDefaultMessageBox ('There is a large difference between the length of your post and the length of the post returned from the internet. Your post has been added as an unposted reply for reference.', 'Warning', mtWarning);
											end else
												ShowDefaultMessageBox ('Your post triggered the "HTML Comments are not allowed" message. It has been added as an unposted reply for reference.', 'Warning', mtWarning);
										end else
											ShowDefaultMessageBox ('The last post in the topic does not contain your own user name. Your post has been added as an unposted reply for reference.', 'Warning', mtWarning);
									end;
							end else
								ShowDefaultMessageBox ('The topic returned from the internet is empty. Your post has been added as an unposted reply for reference.', 'Warning', mtWarning);
						end;
					end;
					if ShowAfterDownload or (Assigned (TopicForm) and ((not ReadAll) or (XLatestChange <> 0))) then
						Show;
				end;
			DownloadTopic := nil;
			UpdatePopupMenu;
		end;
	end;
end;

procedure TMainForm.RedownloadMenuItemClick(Sender: TObject);
var
	CurItem: TListItem;
begin
	CurItem := TopicList.Selected;
	while Assigned (CurItem) do begin
		with CurItem do
			if Assigned (Data) then
				with TTopic (Data) do
					MustDownload := True;
		CurItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
	end;
	SaveTopics;
	UpdatePopupMenu;
end;

procedure TMainForm.UpdatePostCount;
var
	Count: Integer;
	S: string;
begin
	Count := TopicList.Items.Count;
	if Count = 1 then
		S := '1 Topic'
	else
		S := IntToStr (Count) + ' Topics';
	StatusBar.Panels[0].Text := S;
	Count := TotalPosts;
	if Count = 1 then
		S := '1 Post'
	else
		S := IntToStr (Count) + ' Posts';
	StatusBar.Panels[1].Text := S;
	UpdatingPostCount := False;
end;

procedure TMainForm.SetStatusText(const Value: string);
begin
	if FStatusText <> Value then begin
		FStatusText := Value;
		StatusBar.Panels[2].Text := Value;
	end;
end;

procedure TMainForm.RefreshMenuItemClick(Sender: TObject);
begin
	RequestUpdates (HideManualUpdates);
end;

procedure TMainForm.RefreshAllMenuItemClick(Sender: TObject);
begin
	RequestAll;
end;

procedure TMainForm.StopDownloadingMenuItemClick(Sender: TObject);
var
	CurItem: TListItem;
begin
	CurItem := TopicList.Selected;
	while Assigned (CurItem) do begin
		with CurItem do
			if Assigned (Data) then begin
				with TTopic (Data) do
					MustDownload := False;
				if (CurrentAction = raRequestTopic) and (DownloadTopic = TTopic (Data)) then
					CancelAction;
			end;
		CurItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
	end;
	SaveTopics;
	UpdatePopupMenu;
end;

procedure TMainForm.SetDownloadTopic(const Value: TTopic);
begin
	if FDownloadTopic <> Value then begin
		FDownloadTopic := Value;
		ShowAfterDownload := False;
	end;
end;

procedure TMainForm.SettingsButtonClick(Sender: TObject);
var
	I,
	PreviousTimeCorrection: Integer;
begin
	SettingsForm := TSettingsForm.Create (Self);
	with SettingsForm do try
		UseProxyCheckBox.Checked := UseProxy;
		if ProxyPort <> 0 then
			ProxyEdit.Text := Proxy + ':' + IntToStr (ProxyPort);
		GlobalCookieEdit.Text := GlobalCookie;
		if Length (URL) > 0 then
			LocalCookieEdit.Text := LocalCookie
		else begin
			LocalCookieEdit.Enabled := False;
			LocalLoginButton.Enabled := False;
		end;
		CheckForUpdatesCheckBox.Checked := UpdateTimer.Enabled;
		UpdateIntervalEdit.Text := FloatToStr (UpdateTimer.Interval / (1000 * 60));
		HideInitUpdatesCheckBox.Checked := HideInitUpdates;
		HideAutoUpdatesCheckBox.Checked := HideAutoUpdates;
		HideManualUpdatesCheckBox.Checked := HideManualUpdates;
		HideTopicDownloadsCheckBox.Checked := HideTopicDownloads;
		MarkForumsReadCheckBox.Checked := MarkForumsRead;
		UseEZCodesCheckBox.Checked := peEZCodes in PostExtras;
		UseEmoticonsCheckBox.Checked := peEmoticons in PostExtras;
		UseLinkConversionCheckBox.Checked := peAutoLinks in PostExtras;
		UseHTMLCheckBox.Checked := peHTML in PostExtras;
		EmailRepliesCheckBox.Checked := EmailReplies;
		if ButtonsVertical then begin
			if ButtonsLeft then begin
				if ButtonsTop then
					RadioButton8.Checked := True
				else
					RadioButton7.Checked := True;
			end else begin
				if ButtonsTop then
					RadioButton3.Checked := True
				else
					RadioButton4.Checked := True;
			end;
		end else begin
			if ButtonsTop then begin
				if ButtonsLeft then
					RadioButton1.Checked := True
				else
					RadioButton2.Checked := True;
			end else begin
				if ButtonsLeft then
					RadioButton6.Checked := True
				else
					RadioButton5.Checked := True;
			end;
		end;
		ButtonsLeft := RadioButton1.Checked or RadioButton6.Checked or RadioButton7.Checked or RadioButton8.Checked;
		ButtonsTop := RadioButton1.Checked or RadioButton2.Checked or RadioButton3.Checked or RadioButton8.Checked;
		if Length (URL) > 0 then begin
			ForumNameEdit.Text := BoardName;
			TimeCorrectionEdit.Text := IntToStr (TimeCorrection);
		end else begin
			ForumNameEdit.Enabled := False;
			TimeCorrectionEdit.Enabled := False;
		end;
		if ShowModal = mrOK then begin
			UseProxy := UseProxyCheckBox.Checked;
			Proxy := ProxyEdit.Text;
			if Pos (':', Proxy) > 0 then begin
				try
					ProxyPort := StrToInt (Copy (Proxy, Pos (':', Proxy) + 1, Length (Proxy)));
				except end;
				Delete (Proxy, Pos (':', Proxy), Length (Proxy));
			end else
				ProxyPort := 8080;
			GlobalCookie := GlobalCookieEdit.Text;
			try
				UpdateTimer.Interval := Round (StrToFloat (UpdateIntervalEdit.Text) * (1000 * 60));
			except end;
			UpdateTimer.Enabled := CheckForUpdatesCheckBox.Checked;
			HideInitUpdates := HideInitUpdatesCheckBox.Checked;
			HideAutoUpdates := HideAutoUpdatesCheckBox.Checked;
			HideManualUpdates := HideManualUpdatesCheckBox.Checked;
			HideTopicDownloads := HideTopicDownloadsCheckBox.Checked;
			MarkForumsRead := MarkForumsReadCheckBox.Checked;
			PostExtras := [];
			if UseEZCodesCheckBox.Checked then
				Include (PostExtras, peEZCodes);
			if UseEmoticonsCheckBox.Checked then
				Include (PostExtras, peEmoticons);
			if UseLinkConversionCheckBox.Checked then
				Include (PostExtras, peAutoLinks);
			if UseHTMLCheckBox.Checked then
				Include (PostExtras, peHTML);
			EmailReplies := EmailRepliesCheckBox.Checked;
			ButtonsVertical := RadioButton3.Checked or RadioButton4.Checked or RadioButton7.Checked or RadioButton8.Checked;
			ButtonsLeft := RadioButton1.Checked or RadioButton6.Checked or RadioButton7.Checked or RadioButton8.Checked;
			ButtonsTop := RadioButton1.Checked or RadioButton2.Checked or RadioButton3.Checked or RadioButton8.Checked;
			SaveSettings;
			if (LocalCookie <> LocalCookieEdit.Text) or (BoardName <> ForumNameEdit.Text) or ((Length (TimeCorrectionEdit.Text) > 0) and (IntToStr (TimeCorrection) <> TimeCorrectionEdit.Text)) then begin
				PreviousTimeCorrection := TimeCorrection;
				LocalCookie := LocalCookieEdit.Text;
				BoardName := ForumNameEdit.Text;
				try
					TimeCorrection := StrToInt (TimeCorrectionEdit.Text);
				except end;
				SaveTopics;
				if TimeCorrection <> PreviousTimeCorrection then begin
					TimeCorrectionChanged := True;
					try
						with TopicList.Items do begin
							if TimeCorrection > PreviousTimeCorrection then begin
								for I := Count - 1 downto 0 do
									with Item [I] do
										if Assigned (Data) then
											with TTopic (Data) do begin
												Started := Started;
												LatestChange := LatestChange;
											end;
							end else begin
								for I := 0 to Count - 1 do
									with Item [I] do
										if Assigned (Data) then
											with TTopic (Data) do begin
												Started := Started;
												LatestChange := LatestChange;
											end;
							end;
						end;
					finally
						TimeCorrectionChanged := False;
					end;
				end;
			end;
			UpdateButtonPositions;
		end;
	finally
		Free;
		SettingsForm := nil;
	end;
end;

function TMainForm.GetCookieHeader: string;
begin
	if Length (LocalCookie) > 0 then
		Result := 'Cookie: ' + LocalCookie + #13#10
	else if Length (GlobalCookie) > 0 then
		Result := 'Cookie: ' + GlobalCookie + #13#10
	else
		Result := '';
end;

procedure TMainForm.ProcessNewCookies(Doc: string);
var
	S,
	TempS,
	NewCookie: string;
	HadCookies: Boolean;
begin
	HadCookies := False;
	while DeleteToAfterFirst ('Set-Cookie:', Doc, TempS) do begin
		DeleteWhiteSpace (Doc);
		S := Doc;
		if not DeleteToAfterFirst (#13, S, TempS) then
			TempS := S;
		if not DeleteToAfterFirst (#10, TempS, S) then
			S := TempS;
		if Pos (';', S) > 0 then begin
			NewCookie := Trim (Copy (S, 1, Pos (';', S) - 1));
			if Pos ('ezbversion', NewCookie) <= 0 then
				HadCookies := True;
			if Pos ('path=/;', LowerCase (S) + ';') > 0 then begin
				if (Length (GlobalCookie) > 0) or (CurrentAction = raGlobalLogin) then
					SetCookie (GlobalCookie, NewCookie);
				if Assigned (SettingsForm) then
					SettingsForm.GlobalCookieEdit.Text := GlobalCookie;
			end else begin
				if (Length (LocalCookie) > 0) or (CurrentAction = raLocalLogin) then
					SetCookie (LocalCookie, NewCookie);
				if Assigned (SettingsForm) then
					SettingsForm.LocalCookieEdit.Text := LocalCookie;
			end;
		end;
	end;
	if (not HadCookies) and (CurrentAction in [raGlobalLogin, raLocalLogin]) then
		ShowDefaultMessageBox ('No new cookies were returned.', 'Login Failed', mtProgramError);
end;

procedure TMainForm.SetCookie(var CookieStr: string;
	const NewCookie: string);
var
	Ident: string;
	P: Integer;
begin
	if Pos ('=', NewCookie) > 0 then begin
		Ident := Copy (NewCookie, 1, Pos ('=', NewCookie) - 1);
		P := Pos (LowerCase (Ident) + '=', LowerCase (CookieStr));
		if P > 0 then begin
			while (P <= Length (CookieStr)) and (CookieStr [P] <> ';') do
				Delete (CookieStr, P, 1);
			if (P <= Length (CookieStr)) and (CookieStr [P] = ';') then
				Delete (CookieStr, P, 1);
			if (P <= Length (CookieStr)) and (CookieStr [P] = ' ') then
				Delete (CookieStr, P, 1);
			if (Length (CookieStr) > 0) and (CookieStr [Length (CookieStr)] = ' ') then
				Delete (CookieStr, Length (CookieStr), 1);
			if (Length (CookieStr) > 0) and (CookieStr [Length (CookieStr)] = ';') then
				Delete (CookieStr, Length (CookieStr), 1);
		end;
		if Length (CookieStr) > 0 then
			CookieStr := CookieStr + '; ' + NewCookie
		else
			CookieStr := NewCookie;
	end;
end;

procedure TMainForm.AppException(Sender: TObject; E: Exception);
begin
	Inc (InErrorCount);
	try
		ShowDefaultMessageBox ('Internal Error: Exception ' + E.ClassName + ', Message "' + E.Message + '"'#13#10#13#10'Please fill out a bug report form at http://tigcc.ticalc.org/.', 'Internal Error', mtProgramError);
	finally
		Dec (InErrorCount);
	end;
end;

function TMainForm.GetURLPath: string;
begin
	Result := URLID;
	if Length (Result) > 0 then begin
		Result := ExtractFilePath (ParamStr (0)) + Result + '/';
{$IFDEF DEBUGDIR}
		ShowDefaultMessageBox ('Program: ' + ParamStr (0) + #13#10'Path: ' + ExtractFilePath (ParamStr (0)) + #13#10'Result: ' + Result, 'Debug Message');
{$ENDIF}
	end;
end;

function TMainForm.GetURLID: string;
begin
	Result := ExtractURLID (URL);
end;

procedure TMainForm.CheckForCookie;
begin
	if (Length (GlobalCookie) <= 0) and (Length (LocalCookie) <= 0) then begin
		ShowDefaultMessageBox ('You need to set up a cookie to perform this operation.', 'Cookie Needed', mtProgramError);
		Abort;
	end;
end;

procedure TMainForm.NewTopicButtonClick(Sender: TObject);
begin
	CheckForCookie;
	NewTopicForm := TTopicForm.Create (Self);
	with TTopicForm (NewTopicForm) do try
		EditingPost := True;
		EditedPost := TGUIPost.Create (TTopicForm (NewTopicForm), ItemParent);
		with EditedPost do begin
			Row.Height := Row.Height * 2;
			UserIDLabel.Caption := GetAlias (OwnID);
			PostTimeLabel.Caption := FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, Now);
			Title.ReadOnly := False;
			Memo.ReadOnly := False;
			if Assigned (LastSplitter) then begin
				Row.Top := LastSplitter.Top + LastSplitter.Height;
				Splitter.Top := Row.Top + Row.Height;
			end;
			Insert (ItemParent);
			CreatePostIcon;
			if PostIcon.TabOrder < Title.TabOrder then
				PostIcon.TabOrder := Title.TabOrder;
			if Memo.TabOrder < PostIcon.TabOrder then
				Memo.TabOrder := PostIcon.TabOrder;
			CreatePostButton (AddReplyButtonClick);
			ActiveControl := Title;
			LastSplitter := Splitter;
		end;
		ShowModal;
	finally
		if Assigned (Topic) then
			Topic.TopicForm := nil;
		Free;
		NewTopicForm := nil;
	end;
end;

procedure TMainForm.DeleteSelectedTopics;
var
	TopicStr: string;
	CurItem,
	NextItem: TListItem;
begin
	if TopicList.SelCount > 0 then begin
		if TopicList.SelCount = 1 then
			TopicStr := 'topic'
		else
			TopicStr := 'topics';
		if ShowDefaultMessageBox ('Are you sure you want to delete the selected ' + TopicStr + ' from the list? Topics might reappear depending on your board settings.', 'Confirm Deletion', mtQuestion) = idYes then begin
			CurItem := TopicList.Selected;
			while Assigned (CurItem) do begin
				NextItem := TopicList.GetNextItem (CurItem, sdAll, [isSelected]);
				with CurItem do
					if Assigned (Data) then begin
						if (CurrentAction <> raRequestTopic) and (DownloadTopic = Data) then
							CancelAction;
						with TTopic (Data) do begin
							if (Length (FileName) > 0) and FileExists (FileName) then try
								DeleteFile (FileName);
							except end;
							if not Assigned (TopicForm) then
								Free;
						end;
					end;
				CurItem := NextItem;
			end;
			SaveTopics;
		end;
	end;
end;

procedure TMainForm.UpdateTimerTick(Sender: TObject);
begin
	ShouldRequestUpdates := True;
end;

procedure TMainForm.GlobalLogin(const UserName, Password: string);
var
	PostData: string;
begin
	PostData := 'login=' + URLEncode (UserName) + '&password=' + URLEncode (Password);
	DownloadURL := 'http://login.ezboard.com/BBSUser.fromWeb';
	PageURLs := '';
	CurrentAction := raGlobalLogin;
	SendCommand (ExtractHost (DownloadURL), 'POST', DownloadURL, 'Content-Length: ' + IntToStr (Length (PostData)) + #13#10'Content-Type: application/x-www-form-urlencoded'#13#10, PostData);
end;

procedure TMainForm.LocalLogin(const UserName, Password, BoardName: string);
var
	PostData: string;
begin
	PostData := 'boardname=' + URLEncode (BoardName) + '&login=' + URLEncode (UserName) + '&password=' + URLEncode (Password);
	DownloadURL := 'http://' + ExtractHost (URL) + '/BBSUser.fromWeb';
	PageURLs := '';
	CurrentAction := raLocalLogin;
	SendCommand (ExtractHost (DownloadURL), 'POST', DownloadURL, 'Content-Length: ' + IntToStr (Length (PostData)) + #13#10'Content-Type: application/x-www-form-urlencoded'#13#10, PostData);
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
	Update;
	with TopicList do
		if Assigned (Selected) then
			Selected.MakeVisible;
end;

procedure TMainForm.ReloadPostDocReceived;
var
	Doc,
	S,
	TempS: string;
	NewPostIcon: Integer;
begin
	Doc := TextReceived;
	try
		try
			NewPostIcon := 0;
			while DeleteToAfterFirst ('name=posticon', Doc, TempS) do begin
				DeleteToAfterFirst ('>', Doc, S);
				if Pos ('checked', LowerCase (S)) > 0 then
					if DeleteToAfterFirst ('value=', S, TempS) then
						if Length (S) > 0 then begin
							if S [1] = '"' then begin
								Delete (S, 1, 1);
								if Pos ('"', S) > 0 then
									Delete (S, Pos ('"', S), Length (S))
							end else
								if Pos (' ', S) > 0 then
									Delete (S, Pos (' ', S), Length (S));
							if LowerCase (S) <> 'nil' then begin
								try
									NewPostIcon := StrToInt (S);
								except end;
							end;
						end;
			end;
			if not DeleteToAfterFirst ('<textarea', Doc, TempS) then
				Abort;
			if not DeleteToAfterFirst ('>', Doc, TempS) then
				Abort;
			if not DeleteToAfterFirst ('</textarea>', Doc, S) then
				Abort;
			if Assigned (DownloadTopic) and Assigned (DownloadTopic.TopicForm) then
				with TTopicForm (DownloadTopic.TopicForm) do
					if EditingPost and Assigned (EditedPost) then
						with EditedPost do begin
							Memo.Text := DeHTMLize (S);
							if Assigned (PostIcon) then
								PostIcon.ItemIndex := NewPostIcon;
						end;
		except end;
	finally
		CurrentAction := raNone;
		ConnectionStatus := csReady;
		DownloadTopic := nil;
	end;
end;

procedure TMainForm.AddFavorite(const FavName, FavURL: string);
begin
	URLEdit.Items.AddObject (FavName, TStringObject.Create (FavURL));
end;

procedure TMainForm.ClearFavorites;
var
	I: Integer;
begin
	with URLEdit.Items do begin
		for I := Count - 1 downto 0 do
			if Assigned (Objects [I]) then
				with Objects [I] do
					Free;
		Clear;
	end;
end;

procedure TMainForm.URLEditSelect(Sender: TObject);
begin
	ShouldSetURL := True;
end;

procedure TMainForm.TopicListEditing(Sender: TObject; Item: TListItem;
	var AllowEdit: Boolean);
begin
	AllowEdit := True;
	TopicList.OnKeyDown := nil;
end;

procedure TMainForm.TopicListClick(Sender: TObject);
begin
	TopicList.OnKeyDown := TopicListKeyDown;
end;

procedure TMainForm.AliasesButtonClick(Sender: TObject);
var
	I: Integer;
begin
	with TAliasesForm.Create (Self) do try
		with Aliases do
			for I := 0 to Count - 1 do
				with AliasList.Items.Add do begin
					Caption := Names [I];
					SubItems.Add (Values [Caption]);
				end;
		if ShowModal = mrOK then begin
			Aliases.Clear;
			with AliasList.Items do
				for I := 0 to Count - 1 do
					with Item [I] do
						Aliases.Add (Caption + '=' + SubItems [0]);
			SaveAliases;
			UpdateAliases;
		end;
	finally
		Free;
	end;
end;

procedure TMainForm.LoadAliases;
begin
	with TMemIniFile.Create (HomeDir + PrefsFile) do try
		Aliases.Clear;
		ReadSectionValues ('Aliases', Aliases);
	finally
		Free;
	end;
	UpdateAliases;
end;

procedure TMainForm.SaveAliases;
var
	I: Integer;
begin
	with TMemIniFile.Create (HomeDir + PrefsFile) do try
		EraseSection ('Aliases');
		with Aliases do
			for I := 0 to Count - 1 do
				WriteString ('Aliases', Names [I], Values [Names [I]]);
		UpdateFile;
	finally
		Free;
	end;
end;

procedure TMainForm.UpdateAliases;
begin
	LoadTopics;
end;

function TMainForm.GetAlias(const UserID: string): string;
begin
	Result := Aliases.Values [UserID];
	if Length (Result) <= 0 then
		Result := UserID;
end;

procedure TMainForm.UpdateButtonPositions;
begin
	if ButtonsVertical then begin
		if ButtonsTop then
			ButtonPanel.Align := alTop
		else
			ButtonPanel.Align := alBottom;
		if ButtonsLeft then
			ButtonPanePanel.Align := alLeft
		else
			ButtonPanePanel.Align := alRight;
		ButtonPanePanel.Width := SettingsButton.Width + ButtonMargin * 2;
		SettingsButton.Top := ButtonMargin;
		SettingsButton.Left := ButtonMargin;
		AliasesButton.Top := SettingsButton.Top + SettingsButton.Height + ButtonMargin * 2 - 1;
		AliasesButton.Left := ButtonMargin;
		SearchButton.Top := AliasesButton.Top + AliasesButton.Height + ButtonMargin * 2 - 1;
		SearchButton.Left := ButtonMargin;
		NewTopicButton.Top := SearchButton.Top + SearchButton.Height + ButtonMargin * 2 - 1;
		NewTopicButton.Left := ButtonMargin;
		ButtonPanel.Height := NewTopicButton.Top + NewTopicButton.Height + ButtonMargin;
	end else begin
		if ButtonsLeft then
			ButtonPanel.Align := alLeft
		else
			ButtonPanel.Align := alRight;
		if ButtonsTop then
			ButtonPanePanel.Align := alTop
		else
			ButtonPanePanel.Align := alBottom;
		ButtonPanePanel.Height := SettingsButton.Height + ButtonMargin * 2;
		SettingsButton.Left := ButtonMargin;
		SettingsButton.Top := ButtonMargin;
		AliasesButton.Left := SettingsButton.Left + SettingsButton.Width + ButtonMargin * 2 - 1;
		AliasesButton.Top := ButtonMargin;
		SearchButton.Left := AliasesButton.Left + AliasesButton.Width + ButtonMargin * 2 - 1;
		SearchButton.Top := ButtonMargin;
		NewTopicButton.Left := SearchButton.Left + SearchButton.Width + ButtonMargin * 2 - 1;
		NewTopicButton.Top := ButtonMargin;
		ButtonPanel.Width := NewTopicButton.Left + NewTopicButton.Width + ButtonMargin;
	end;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
	CanClose := True;
	if NewPostCount > 0 then begin
		if ShowDefaultMessageBox ('This forum contains unverified new posts. Are you sure you want to exit the program?', 'Warning', mtQuestion) = idNo then
			CanClose := False;
	end;
end;

procedure TMainForm.LoadFavorites;
var
	I: Integer;
	Favorite: string;
	URLs: TStringList;
begin
	with TMemIniFile.Create (HomeDir + PrefsFile) do try
		ClearFavorites;
		URLs := TStringList.Create;
		try
			ReadSection ('Favorites', URLs);
			with URLs do
				for I := 0 to Count - 1 do
					if Length (Strings [I]) > 0 then begin
{$IFDEF DEBUGDIR}
						ShowDefaultMessageBox ('Program: ' + ParamStr (0) + #13#10'Path: ' + ExtractFilePath (ParamStr (0)) + #13#10'Result: ' + ExtractFilePath (ParamStr (0)) + ExtractURLID (Strings [I]) + '\' + ListFile, 'Debug Message');
{$ENDIF}
						if FileExists (ExtractFilePath (ParamStr (0)) + ExtractURLID (Strings [I]) + '/' + ListFile) then begin
							Favorite := ReadString ('Favorites', Strings [I], '');
							if Length (Favorite) > 0 then
								AddFavorite (Favorite, Strings [I]);
						end;
					end;
		finally
			URLs.Free;
		end;
	finally
		Free;
	end;
end;

procedure TMainForm.TopicListColumnClick(Sender: TObject;
  Column: TListColumn);
begin
	with TopicList do begin
		if SortColumn = Column.Index then begin
			if SortDirection = sdAscending then
				SortDirection := sdDescending
			else
				SortDirection := sdAscending;
		end else begin
			SortColumn := Column.Index;
			SortDirection := sdAscending;
		end;
	end;
	SaveSettings;
end;

procedure TMainForm.ExecProcTimerTick(Sender: TObject);
var
	I: Integer;
begin
	for I := Low (ExecProcArray) to High (ExecProcArray) do
		ExecProcArray [I];
	SetLength (ExecProcArray, 0);
end;

end.
