unit TopicUnit;

interface

uses
	MasterUnit,
	SysUtils, Classes, QForms, QStdCtrls, QComCtrls;

type
	TTopic = class(TObject)
	private
		FRelURL: string;
		FTopicID: string;
		FListItem: TListItem;
		FTopic: string;
		FReplies: Integer;
		FLatestChange: TDateTime;
		FStartedBy: string;
		FLastComment: string;
		FReadReplies: Integer;
		FReadFirst: Boolean;
		FUserEdited: Boolean;
		FMustDownload: Boolean;
		FTopicContents: TCollection;
		FStarted: TDateTime;
		FTopicForm: TForm;
		procedure SetTopic(const Value: string);
		procedure SetLastComment(const Value: string);
		procedure SetLatestChange(const Value: TDateTime);
		procedure SetReplies(const Value: Integer);
		procedure SetStartedBy(const Value: string);
		procedure SetReadReplies(const Value: Integer);
		procedure SetReadFirst(const Value: Boolean);
		function GetReadAll: Boolean;
		procedure SetReadAll(const Value: Boolean);
		procedure SetMustDownload(const Value: Boolean);
		function GetFileName: string;
		procedure SetRelURL(const Value: string);
		function GetTopicContents: TCollection;
		procedure SetStarted(const Value: TDateTime);
		function GetReadPosts: Integer;
		function GetContainsDeleted: Boolean;
    function GetAbsURL: string;
	public
		constructor Create;
		destructor Destroy; override;
		procedure UpdateReplies;
		procedure Show;
		property ListItem: TListItem read FListItem write FListItem;
		property RelURL: string read FRelURL write SetRelURL;
		property AbsURL: string read GetAbsURL;
		property TopicName: string read FTopic write SetTopic;
		property Replies: Integer read FReplies write SetReplies;
		property ReadFirst: Boolean read FReadFirst write SetReadFirst;
		property ReadReplies: Integer read FReadReplies write SetReadReplies;
		property ReadAll: Boolean read GetReadAll write SetReadAll;
		property ReadPosts: Integer read GetReadPosts;
		property StartedBy: string read FStartedBy write SetStartedBy;
		property LastComment: string read FLastComment write SetLastComment;
		property Started: TDateTime read FStarted write SetStarted;
		property LatestChange: TDateTime read FLatestChange write SetLatestChange;
		property UserEdited: Boolean read FUserEdited write FUserEdited;
		property MustDownload: Boolean read FMustDownload write SetMustDownload;
		property FileName: string read GetFileName;
		property ContainsDeleted: Boolean read GetContainsDeleted;
		property TopicContents: TCollection read GetTopicContents;
		property TopicForm: TForm read FTopicForm write FTopicForm;
	end;

implementation

uses
	TopicContentsUnit,
	MainUnit, TopicWinUnit, SearchUnit;

{ TTopic }

constructor TTopic.Create;
begin
	inherited;
	FReplies := -1;
	FReadReplies := -1;
end;

destructor TTopic.Destroy;
var
	I,
	J: Integer;
begin
	if MainForm.DownloadTopic = Self then begin
		MainForm.CancelAction;
		MainForm.DownloadTopic := nil;
	end;
	if Assigned (FListItem) then begin
		FListItem.Data := nil;
		FListItem.Free;
		FListItem := nil;
	end;
	if Assigned (FTopicContents) then begin
		FTopicContents.Free;
		FTopicContents := nil;
	end;
	MustDownload := False;
	Replies := -1;
	with MainForm do
		for I := 0 to ComponentCount - 1 do
			if Components [I] is TSearchForm then
				with Components [I] as TSearchForm do
					with ResultsList.Items do
						for J := Count - 1 downto 0 do
							with Item [J] do
								if Data = Self then
									Delete;
	inherited;
end;

function TTopic.GetAbsURL: string;
begin
	Result := 'http://' + ExtractHost (MainForm.URL) + RelURL;
end;

function TTopic.GetContainsDeleted: Boolean;
begin
	Result := TTopicContents(TopicContents).ContainsDeleted;
end;

function TTopic.GetFileName: string;
begin
	if Length (FTopicID) > 0 then
		Result := MainForm.URLPath + FTopicID + '.brt'
	else
		Result := '';
end;

function TTopic.GetReadAll: Boolean;
begin
	Result := ReadFirst and (ReadReplies >= Replies);
end;

function TTopic.GetReadPosts: Integer;
begin
	if ReadFirst then
		Result := ReadReplies + 1
	else
		Result := 0;
end;

function TTopic.GetTopicContents: TCollection;
begin
	if not Assigned (FTopicContents) then begin
		FTopicContents := TTopicContents.Create (Self);
		TTopicContents(FTopicContents).Load;
	end;
	Result := FTopicContents;
end;

procedure TTopic.SetLastComment(const Value: string);
begin
	if FLastComment <> Value then begin
		FLastComment := Value;
		with MainForm do
			AddSubItem (ListItem, LastCommentColumn, GetAlias (Value));
	end;
end;

procedure TTopic.SetLatestChange(const Value: TDateTime);
begin
	if (FLatestChange <> Value) or MainForm.TimeCorrectionChanged then begin
		FLatestChange := Value;
		with MainForm do
			AddSubItem (ListItem, LatestChangeColumn, FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, Value + TimeCorrection / 24));
		if (Started > LatestChange) or (Started = 0) then
			Started := LatestChange;
	end;
end;

procedure TTopic.SetMustDownload(const Value: Boolean);
begin
	if FMustDownload <> Value then begin
		FMustDownload := Value;
		if Value then
			Inc (MainForm.TopicsToDownload)
		else
			Dec (MainForm.TopicsToDownload);
	end;
end;

procedure TTopic.SetReadAll(const Value: Boolean);
begin
	if ReadAll <> Value then begin
		ReadFirst := Value;
		if Value then
			ReadReplies := Replies
		else
			ReadReplies := 0;
		with MainForm do
			if not UpdatingPopup then begin
				UpdatingPopup := True;
				ExecProc (UpdatePopupMenu);
			end;
	end;
end;

procedure TTopic.SetReadFirst(const Value: Boolean);
begin
	if FReadFirst <> Value then begin
		FReadFirst := Value;
		UpdateReplies;
	end;
end;

procedure TTopic.SetReadReplies(const Value: Integer);
begin
	if FReadReplies <> Value then begin
		FReadReplies := Value;
		UpdateReplies;
	end;
end;

procedure TTopic.SetRelURL(const Value: string);
var
	P: Integer;
begin
	if FRelURL <> Value then begin
		FRelURL := Value;
		FTopicID := Value;
		if Length (FTopicID) > 0 then begin
			while Pos ('/', FTopicID) > 0 do
				Delete (FTopicID, 1, Pos ('/', FTopicID));
			P := Pos ('?', FTopicID);
			if P > 0 then begin
				FTopicID [P] := '-';
				FTopicID := StringReplace (FTopicID, '.ShowMessage', '', [rfReplaceAll, rfIgnoreCase]);
				FTopicID := StringReplace (FTopicID, 'topicID=', '', [rfReplaceAll, rfIgnoreCase]);
				FTopicID := StringReplace (FTopicID, '.topic', '', [rfReplaceAll, rfIgnoreCase]);
				FTopicID := StringReplace (FTopicID, '=', '_', [rfReplaceAll]);
				FTopicID := StringReplace (FTopicID, '&', '-', [rfReplaceAll]);
			end else
				FTopicID := '';
		end;
	end;
end;

procedure TTopic.SetReplies(const Value: Integer);
begin
	if FReplies <> Value then begin
		with MainForm do begin
			Inc (TotalPosts, Value - FReplies);
			if not UpdatingPostCount then begin
				UpdatingPostCount := True;
				ExecProc (UpdatePostCount);
			end;
		end;
		FReplies := Value;
		UpdateReplies;
	end;
end;

procedure TTopic.SetStarted(const Value: TDateTime);
begin
	if ((FStarted <> Value) or MainForm.TimeCorrectionChanged) and (Value <= LatestChange) then begin
		FStarted := Value;
		with MainForm do
			AddSubItem (ListItem, StartedColumn, FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, Value + TimeCorrection / 24));
	end;
end;

procedure TTopic.SetStartedBy(const Value: string);
begin
	if FStartedBy <> Value then begin
		FStartedBy := Value;
		with MainForm do
			AddSubItem (ListItem, StartedByColumn, GetAlias (Value));
	end;
end;

procedure TTopic.SetTopic(const Value: string);
begin
	if FTopic <> Value then begin
		FTopic := Value;
		with MainForm do
			AddSubItem (ListItem, TopicColumn, Value);
	end;
end;

procedure TTopic.Show;
var
	Read: Integer;
begin
	if MustDownload then begin
		with MainForm do begin
			if not ((CurrentAction = raRequestTopic) and (DownloadTopic = Self)) then
				RequestTopic (Self);
			ShowAfterDownload := True;
		end;
	end else begin
		Read := ReadPosts;
		if not ReadAll then begin
			ReadAll := True;
			MainForm.SaveTopics;
		end;
		if Assigned (FTopicForm) then begin
			with TTopicForm (FTopicForm) do
				if Topic = Self then
					UpdatePosts
				else begin
					ReadPosts := Read;
					Topic := Self;
				end;
		end else begin
			FTopicForm := TTopicForm.Create (MainForm);
			try
				with TTopicForm (FTopicForm) do try
					ReadPosts := Read;
					Topic := Self;
					ShowModal;
				finally
					Free;
				end;
			finally
				FTopicForm := nil;
				with MainForm do
					if (CurrentAction = raReloadPost) and (DownloadTopic = Self) then
						CancelAction;
			end;
		end;
	end;
end;

procedure TTopic.UpdateReplies;
var
	S: string;
begin
	S := IntToStr (Replies);
	if ReadFirst and (ReadReplies < Replies) then
		S := S + ' (' + IntToStr (ReadReplies) + ')';
	with MainForm do
		AddSubItem (ListItem, RepliesColumn, S);
	if Assigned (ListItem) then
		with ListItem do begin
			if ReadAll then
				ImageIndex := 1
			else begin
				ImageIndex := 0;
			end;
		end;
end;

end.
