unit TopicWinUnit;

interface

uses
	MasterUnit,
	TopicUnit, TopicContentsUnit,
	SysUtils, Classes, QGraphics, QControls, QForms, QDialogs,
	QStdCtrls, QExtCtrls, QImgList, QButtons, QComCtrls;

type
	TGUIPost = class;

	TTopicForm = class(TForm)
		ScrollBox: TScrollBox;
		HorzPanel: TPanel;
    ContainsDeletedIndicator: TLabel;
		ExtraImages: TImageList;
    MoveUpButton: TSpeedButton;
		MoveDownButton: TSpeedButton;
    ButtonPanel: TPanel;
    AddReplyButton: TButton;
    SettingsButton: TButton;
    ShowPageButton: TButton;
    VertPanel: TPanel;
		procedure FormCreate(Sender: TObject);
		procedure FormKeyDown(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure FormActivate(Sender: TObject);
		procedure ShowPageButtonClick(Sender: TObject);
		procedure AddReplyButtonClick(Sender: TObject);
		procedure SettingsButtonClick(Sender: TObject);
		procedure FormClose(Sender: TObject; var Action: TCloseAction);
		procedure EditButtonClick(Sender: TObject);
		procedure DeleteButtonClick(Sender: TObject);
		procedure MoveUpButtonClick(Sender: TObject);
		procedure MoveDownButtonClick(Sender: TObject);
		procedure TopicLinkClick(Sender: TObject);
		procedure ChangeTopic(Sender: TObject);
	private
		FTopic: TTopic;
		FReadPosts: Integer;
		FItemParent: TWinControl;
		FEditingPost: Boolean;
		FCanUpdate: Boolean;
		FChangeTopic: TTopic;
		procedure SetTopic(const Value: TTopic);
		procedure SetReadPosts(const Value: Integer);
		procedure SetEditingPost(const Value: Boolean);
	public
		HasUnread: Boolean;
		LastSplitter: TControl;
		LastTitle: string;
		EditedPost: TGUIPost;
		FormShown: Boolean;
		PostExtras: TPostExtras;
		EmailReplies: Boolean;
		procedure ClearPosts;
		procedure UpdatePosts;
		procedure ScrollDown;
		property Topic: TTopic read FTopic write SetTopic;
		property ReadPosts: Integer read FReadPosts write SetReadPosts;
		property ItemParent: TWinControl read FItemParent;
		property EditingPost: Boolean read FEditingPost write SetEditingPost;
		property CanUpdate: Boolean read FCanUpdate write FCanUpdate;
	end;

	TGUIPost = class(TObject)
		procedure SplitterMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure SplitterMouseMove(Sender: TObject; Shift: TShiftState; X,
			Y: Integer);
		procedure SplitterMouseUp(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
	private
		SplitterMoving: Boolean;
	public
		PostExists: Boolean;
		ReplyIndex: Integer;
		PostExtras: TPostExtras;
		Row: TPanel;
		LeftCol: TPanel;
		RightCol: TPanel;
		UserIDLabel: TLabel;
		PostTimeLabel: TLabel;
		Title: TEdit;
		Memo: TMemo;
		PostIcon: TComboBox;
		EditButton: TButton;
		DeleteButton: TButton;
		PostButton: TButton;
		Splitter: TPanel;
		constructor Create(Sender: TTopicForm; ItemOwner: TComponent; LoadFromPost: TTopicPost = nil; Index: Integer = 0; IsRead: Boolean = False);
		destructor Destroy; override;
		procedure Insert(Control: TWinControl);
		procedure CreatePostIcon(IconIndex: Integer = 0);
		procedure CreateEditButton(OnClickEvent: TNotifyEvent);
		procedure CreateDeleteButton(OnClickEvent: TNotifyEvent);
		procedure CreatePostButton(OnClickEvent: TNotifyEvent);
	end;

implementation

uses
	LibC,
	MainUnit, PostSettingsUnit,
	UtilsWin,
	URLLabelUnit;

{$R *.dfm}

{ TGUIPost }

constructor TGUIPost.Create(Sender: TTopicForm; ItemOwner: TComponent; LoadFromPost: TTopicPost; Index: Integer; IsRead: Boolean);
var
	I: Integer;
	CurURLLabel: TURLLabel;
	Link: string;
	LinkTopicItem: TListItem;
	LinkTopic: TTopic;
begin
	inherited Create;
	PostExists := Assigned (LoadFromPost);
	if Assigned (LoadFromPost) then begin
		ReplyIndex := LoadFromPost.PostIndex;
		PostExtras := LoadFromPost.PostExtras;
	end else
		ReplyIndex := Index;
	Row := TPanel.Create (ItemOwner);
	with Row do begin
		Align := alTop;
		Height := 100;
		Caption := '';
		BevelInner := bvNone;
		BevelOuter := bvNone;
		ParentColor := True;
		Tag := Integer (Self);
	end;
	LeftCol := TPanel.Create (ItemOwner);
	with LeftCol do begin
		Align := alLeft;
		Width := 125;
		Caption := '';
		BevelInner := bvNone;
		BevelOuter := bvLowered;
		ParentColor := True;
		Parent := Row;
	end;
	UserIDLabel := TLabel.Create (ItemOwner);
	with UserIDLabel do begin
		Left := 4;
		Top := 4;
		if Assigned (LoadFromPost) then
			Caption := MainForm.GetAlias (LoadFromPost.UserID);
		Parent := LeftCol;
	end;
	PostTimeLabel := TLabel.Create (ItemOwner);
	with PostTimeLabel do begin
		Left := 4;
		Top := 20;
		if Assigned (LoadFromPost) then
			Caption := FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, LoadFromPost.PostTime + MainForm.TimeCorrection / 24);
		Parent := LeftCol;
	end;
	if Assigned (LoadFromPost) and (Length (LoadFromPost.EditorID) > 0) then begin
		with TLabel.Create (ItemOwner) do begin
			Left := 4;
			Top := 40;
			Caption := 'Edited:';
			Parent := LeftCol;
		end;
		with TLabel.Create (ItemOwner) do begin
			Left := 4;
			Top := 56;
			Caption := FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, LoadFromPost.EditTime + MainForm.TimeCorrection / 24);
			Parent := LeftCol;
		end;
		if LoadFromPost.EditorID <> LoadFromPost.UserID then
			with TLabel.Create (ItemOwner) do begin
				Left := 4;
				Top := 72;
				Caption := '(' + MainForm.GetAlias (LoadFromPost.EditorID) + ')';
				Parent := LeftCol;
			end;
	end;
	if peEZCodes in PostExtras then
		with TImage.Create (LeftCol) do begin
			Left := LeftCol.Width - 16;
			Top := 24;
			Width := 8;
			Height := 8;
			Stretch := False;
			Center := False;
			Sender.ExtraImages.GetBitmap (0, Picture.Bitmap);
			Parent := LeftCol;
		end;
	if peEmoticons in PostExtras then
		with TImage.Create (LeftCol) do begin
			Left := LeftCol.Width - 16;
			Top := 32;
			Width := 8;
			Height := 8;
			Stretch := False;
			Center := False;
			Sender.ExtraImages.GetBitmap (1, Picture.Bitmap);
			Parent := LeftCol;
		end;
	RightCol := TPanel.Create (ItemOwner);
	with RightCol do begin
		Align := alClient;
		Caption := '';
		BevelInner := bvNone;
		BevelOuter := bvLowered;
		ParentColor := True;
		Parent := Row;
	end;
	Title := TEdit.Create (ItemOwner);
	with Title do begin
		Align := alTop;
		ReadOnly := True;
		if Assigned (LoadFromPost) then
			Text := LoadFromPost.Title;
		Font.Style := Font.Style + [fsBold];
		Color := clWindow;
		Parent := RightCol;
	end;
	Memo := TMemo.Create (ItemOwner);
	with Memo do begin
		Align := alClient;
		if Assigned (LoadFromPost) then
			Text := LoadFromPost.Text;
		WordWrap := True;
		ReadOnly := True;
		BorderStyle := bsNone;
		ScrollBars := ssVertical;
		if IsRead then
			ParentColor := True
		else
			Color := clWindow;
		Parent := RightCol;
	end;
	if Assigned (LoadFromPost) then
		for I := 0 to LoadFromPost.LinkURLs.Count - 1 do begin
			with TPanel.Create (ItemOwner) do begin
				Align := alBottom;
				Height := 1;
				Top := RightCol.Height - Height;
				Caption := '';
				BevelInner := bvNone;
				BevelOuter := bvNone;
				Color := clBtnShadow;
				Parent := RightCol;
			end;
			Link := LoadFromPost.LinkURLs.Strings [I];
			LinkTopic := nil;
			if Pos ('ezboard.com', Link) > 0 then begin
				LinkTopicItem := MainForm.FindTopicByURL (Link);
				if Assigned (LinkTopicItem) then begin
					LinkTopic := LinkTopicItem.Data;
					if Assigned (LinkTopic) then
						Link := LinkTopic.TopicName;
				end;
			end;
			CurURLLabel := TURLLabel.Create (ItemOwner);
			with CurURLLabel do begin
				AutoSize := True;
				if Assigned (LinkTopic) then begin
					Tag := Integer (LinkTopic);
					OnClick := Sender.TopicLinkClick;
				end else
					URL := Link;
				Caption := StringReplace (Link, '&', '&&', [rfReplaceAll]);
				AutoSize := False;
				Align := alBottom;
				Row.Height := Row.Height + Height + 1;
				Top := RightCol.Height - Height;
				Parent := RightCol;
			end;
		end;
	if Assigned (LoadFromPost) then begin
		if LoadFromPost.CanEdit then
			CreateEditButton (Sender.EditButtonClick);
		if LoadFromPost.CanDelete then
			CreateDeleteButton (Sender.DeleteButtonClick);
	end;
	Splitter := TPanel.Create (ItemOwner);
	with Splitter do begin
		Align := alTop;
		Height := 3;
		Top := Row.Top + Row.Height;
		Caption := '';
		BevelInner := bvNone;
		BevelOuter := bvNone;
		ParentColor := True;
		Cursor := crVSplit;
		OnMouseDown := SplitterMouseDown;
		OnMouseMove := SplitterMouseMove;
		OnMouseUp := SplitterMouseUp;
	end;
end;

procedure TGUIPost.CreateDeleteButton(OnClickEvent: TNotifyEvent);
begin
	if not Assigned (DeleteButton) then begin
		DeleteButton := TButton.Create (Row);
		with DeleteButton do begin
			Anchors := [akRight, akBottom];
			Left := LeftCol.Width div 2 + 4;
			Width := LeftCol.Width div 2 - 12;
			Height := Height * 4 div 5;
			Top := LeftCol.Height - Height - 4;
			Caption := 'Delete';
			Tag := Integer (Self);
			OnClick := OnClickEvent;
			Parent := LeftCol;
		end;
	end;
end;

procedure TGUIPost.CreateEditButton(OnClickEvent: TNotifyEvent);
begin
	if not Assigned (EditButton) then begin
		EditButton := TButton.Create (Row);
		with EditButton do begin
			Anchors := [akLeft, akBottom];
			Left := 8;
			Width := LeftCol.Width div 2 - 12;
			Height := Height * 4 div 5;
			Top := LeftCol.Height - Height - 4;
			Caption := 'Edit';
			Tag := Integer (Self);
			OnClick := OnClickEvent;
			Parent := LeftCol;
		end;
	end;
end;

procedure TGUIPost.CreatePostButton(OnClickEvent: TNotifyEvent);
begin
	if Assigned (DeleteButton) then begin
		DeleteButton.Free;
		DeleteButton := nil;
	end;
	if not Assigned (PostButton) then begin
		PostButton := TButton.Create (Row);
		with PostButton do begin
			if Assigned (EditButton) then begin
				Anchors := [akRight, akBottom];
				Left := LeftCol.Width div 2 + 4;
				Width := LeftCol.Width div 2 - 12;
				Height := Height * 4 div 5;
				Top := LeftCol.Height - Height - 4;
			end else begin
				Anchors := [akLeft, akRight, akBottom];
				Left := 10;
				Width := LeftCol.Width - 20;
				Top := LeftCol.Height - Height - 10;
			end;
			Caption := 'Post';
			Tag := Integer (Self);
			OnClick := OnClickEvent;
			Parent := LeftCol;
		end;
	end;
end;

procedure TGUIPost.CreatePostIcon(IconIndex: Integer);
var
	TitleBack: TPanel;
begin
	if not Assigned (PostIcon) then begin
		TitleBack := TPanel.Create (Row);
		with TitleBack do begin
			Align := alTop;
			Height := Title.Height;
			Caption := '';
			BevelInner := bvNone;
			BevelOuter := bvNone;
			ParentColor := True;
			Parent := RightCol;
		end;
		Title.Parent := TitleBack;
		Title.Align := alClient;
		PostIcon := TComboBox.Create (Row);
		with PostIcon do begin
			Align := alRight;
			Width := 90;
			Style := csDropDownList;
			Parent := TitleBack;
			Items.Add ('No Icon');
			Items.Add ('Down Arrow');
			Items.Add ('Tear');
			Items.Add ('Important');
			Items.Add ('Light Bulb');
			Items.Add ('Angry Face');
			Items.Add ('Question');
			Items.Add ('Smiley');
			Items.Add ('Sunglasses');
			Items.Add ('Thumb Down');
			Items.Add ('Thumb Up');
			ItemIndex := IconIndex;
			if TitleBack.Height < Height then
				TitleBack.Height := Height;
		end;
	end;
end;

destructor TGUIPost.Destroy;
begin
	if Assigned (Row) then
		Row.Tag := 0;
	if Assigned (EditButton) then
		EditButton.Tag := 0;
	if Assigned (DeleteButton) then
		DeleteButton.Tag := 0;
	if Assigned (PostButton) then
		PostButton.Tag := 0;
	inherited;
end;

procedure TGUIPost.Insert(Control: TWinControl);
begin
	Row.Parent := Control;
	Splitter.Parent := Control;
end;

procedure TGUIPost.SplitterMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if Button = mbLeft then
		SplitterMoving := True;
end;

procedure TGUIPost.SplitterMouseMove(Sender: TObject;
	Shift: TShiftState; X, Y: Integer);
begin
	if SplitterMoving then begin
		with Row do
			Height := Height + Y;
		Row.Update;
	end;
end;

procedure TGUIPost.SplitterMouseUp(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	if Button = mbLeft then
		SplitterMoving := False;
end;

{ TTopicForm }

procedure TTopicForm.FormCreate(Sender: TObject);
begin
	CanUpdate := True;
	FItemParent := ScrollBox;
	if MainForm.TopicWinWidth <= 0 then begin
		Left := MainForm.Left + 10;
		Top := MainForm.Top + 10;
		Width := MainForm.Width - 20;
		Height := MainForm.Height - 20;
	end else begin
		Left := MainForm.TopicWinLeft;
		Top := MainForm.TopicWinTop;
		Width := MainForm.TopicWinWidth;
		Height := MainForm.TopicWinHeight;
	end;
	PostExtras := MainForm.PostExtras;
	EmailReplies := MainForm.EmailReplies;
	if MainForm.ButtonsVertical then begin
		ButtonPanel.Parent := VertPanel;
		if MainForm.ButtonsTop then
			ButtonPanel.Align := alTop
		else
			ButtonPanel.Align := alBottom;
		if MainForm.ButtonsLeft then
			VertPanel.Align := alLeft
		else
			VertPanel.Align := alRight;
		VertPanel.Width := ShowPageButton.Width + ButtonMargin * 2;
		ShowPageButton.Top := ButtonMargin;
		ShowPageButton.Left := ButtonMargin;
		SettingsButton.Top := ShowPageButton.Top + ShowPageButton.Height + ButtonMargin * 2 - 1;
		SettingsButton.Left := ButtonMargin;
		AddReplyButton.Top := SettingsButton.Top + SettingsButton.Height + ButtonMargin * 2 - 1;
		AddReplyButton.Left := ButtonMargin;
		ButtonPanel.Height := AddReplyButton.Top + AddReplyButton.Height + ButtonMargin;
		VertPanel.Show;
	end else begin
		ButtonPanel.Parent := HorzPanel;
		if MainForm.ButtonsTop then
			HorzPanel.Align := alTop
		else
			HorzPanel.Align := alBottom;
		HorzPanel.Height := ShowPageButton.Height + ButtonMargin * 2;
		ShowPageButton.Left := ButtonMargin;
		ShowPageButton.Top := ButtonMargin;
		SettingsButton.Left := ShowPageButton.Left + ShowPageButton.Width + ButtonMargin * 2 - 1;
		SettingsButton.Top := ButtonMargin;
		AddReplyButton.Left := SettingsButton.Left + SettingsButton.Width + ButtonMargin * 2 - 1;
		AddReplyButton.Top := ButtonMargin;
		ButtonPanel.Width := AddReplyButton.Left + AddReplyButton.Width + ButtonMargin;
		VertPanel.Hide;
	end;
end;

procedure TTopicForm.SetReadPosts(const Value: Integer);
begin
	if FReadPosts <> Value then begin
		FReadPosts := Value;
		UpdatePosts;
	end;
end;

procedure TTopicForm.SetTopic(const Value: TTopic);
begin
	if FTopic <> Value then begin
		FTopic := Value;
		UpdatePosts;
		ShowPageButton.Enabled := Assigned (Value);
	end;
end;

procedure TTopicForm.UpdatePosts;
var
	I: Integer;
	Item: TTopicPost;
	Read: Boolean;
begin
	if CanUpdate then begin
		ClearPosts;
		with MainForm.TopicList do
			if Assigned (Selected) then
				with Selected do begin
					MoveUpButton.Enabled := Index > 0;
					MoveDownButton.Enabled := Index < Items.Count - 1;
				end;
		if Assigned (Topic) then begin
			with Topic do begin
				Self.Caption := 'Topic - ' + TopicName;
				ContainsDeletedIndicator.Visible := ContainsDeleted;
				with TTopicContents (TopicContents) do begin
					for I := 0 to Count - 1 do begin
						Item := TTopicPost (Items [I]);
						LastTitle := Item.Title;
						Read := I < Self.ReadPosts;
						if not Read then
							HasUnread := True;
						with TGUIPost.Create (Self, ItemParent, Item, I, Read) do begin
							if Assigned (LastSplitter) then begin
								Row.Top := LastSplitter.Top + LastSplitter.Height;
								Splitter.Top := Row.Top + Row.Height;
							end;
							Insert (ItemParent);
							LastSplitter := Splitter;
						end;
					end;
					if Length (NewPost) > 0 then begin
						AddReplyButtonClick (AddReplyButton);
						if Assigned (EditedPost) then
							with EditedPost do begin
								Title.Text := NewPostTitle;
								Memo.Text := NewPost;
							end;
					end;
				end;
			end;
			if FormShown and (ReadPosts > 0) and HasUnread then
				ScrollDown;
		end;
	end;
end;

procedure TTopicForm.FormKeyDown(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	if Key = 4096 then
		ModalResult := mrCancel;
end;

procedure TTopicForm.FormActivate(Sender: TObject);
begin
	FormShown := True;
	if (ReadPosts > 0) and HasUnread then
		ScrollDown;
end;

procedure TTopicForm.ShowPageButtonClick(Sender: TObject);
var
	ArgList: array [0..2] of PChar;
begin
	if Assigned (Topic) then begin
		if LibC.Fork = 0 then begin
			ArgList [0] := 'x-www-browser';
			ArgList [1] := PChar (Topic.AbsURL);
			ArgList [2] := nil;
			LibC.ExecVP ('x-www-browser', PPChar (@ArgList));
			Halt;
		end;
	end;
end;

procedure TTopicForm.SetEditingPost(const Value: Boolean);
procedure CleanupControls(ControlParent: TWinControl);
var
	I: Integer;
begin
	with ControlParent do
		for I := ControlCount - 1 downto 0 do
			if Controls [I] is TButton then begin
				with TButton (Controls [I]) do
					if (Pos ('edit', LowerCase (Caption)) > 0) or (Pos ('del', LowerCase (Caption)) > 0) then begin
						if Tag <> 0 then
							with TGUIPost (Tag) do begin
								if EditButton = ControlParent.Controls [I] then
									EditButton := nil;
								if DeleteButton = ControlParent.Controls [I] then
									DeleteButton := nil;
							end;
						Free;
					end;
			end else if Controls [I] is TWinControl then
				CleanupControls (TWinControl (Controls [I]));
end;
begin
	if FEditingPost <> Value then begin
		FEditingPost := Value;
		if Value then begin
			AddReplyButton.Caption := '&Post';
		end else
			AddReplyButton.Caption := 'Add &Reply';
		if Value then begin
			CleanupControls (ItemParent);
			CanUpdate := False;
		end;
	end;
end;

procedure TTopicForm.ClearPosts;
begin
	ItemParent.Hide;
	try
		EditingPost := False;
		EditedPost := nil;
		LastSplitter := nil;
		HasUnread := False;
		with ItemParent do
			while (ComponentCount) > 0 do
				with Components [0] do begin
					if Tag <> 0 then
						TGUIPost(Tag).Free;
					Free;
				end;
		MoveUpButton.Enabled := False;
		MoveDownButton.Enabled := False;
	finally
		ItemParent.Show;
	end;
end;

procedure TTopicForm.ScrollDown;
begin
	if Assigned (LastSplitter) then
		ScrollBox.ScrollInView (LastSplitter);
end;

procedure TTopicForm.AddReplyButtonClick(Sender: TObject);
var
	PostData,
	PostIconStr: string;
function CheckForHTMLComments(const HTMLCommentString: string): Boolean;
var
	I,
	StartI,
	CurPos: Integer;
begin
	Result := Pos (HTMLCommentString, StringReplace (PostData, ' ', '', [rfReplaceAll])) > 0;
	if Result then begin
		CurPos := 1;
		StartI := 1;
		for I := 1 to Length (PostData) do
			if PostData [I] = HTMLCommentString [CurPos] then begin
				if CurPos = 1 then
					StartI := I;
				Inc (CurPos);
				if CurPos > Length (HTMLCommentString) then begin
					EditedPost.Memo.SelStart := StartI - 1;
					EditedPost.Memo.SelLength := I - StartI + 1;
					try
						ActiveControl := EditedPost.Memo;
					except end;
				end;
			end else if PostData [I] <> ' ' then
				CurPos := 1;
	end;
end;
begin
	MainForm.CheckForCookie;
	if EditingPost then begin
		if Assigned (EditedPost) then
			with EditedPost do begin
				PostData := Memo.Text;
				if (Length (Title.Text) > 0) and (Length (PostData) > 0) then begin
					if CheckForHTMLComments ('<!--') or CheckForHTMLComments ('-->') then
						ShowDefaultMessageBox ('Everything looking like an HTML comment is rejected.', 'Error', mtProgramError)
					else begin
						if Assigned (Topic) and (not PostExists) then
							with TTopicContents (Topic.TopicContents) do begin
								NewPost := PostData;
								NewPostTitle := Title.Text;
							end;
						if PostIcon.ItemIndex = 0 then
							PostIconStr := 'nil'
						else
							PostIconStr := IntToStr (PostIcon.ItemIndex);
						with MainForm do begin
							PostData := 'posticon=' + PostIconStr + '&subject=' + URLEncode (Title.Text) + '&body=' + URLEncode (AdjustLineBreaks (Memo.Text, tlbsCRLF)) + '&forumName=' + URLID;
							if peEZCodes in Self.PostExtras then
								PostData := PostData + '&postType=ezcodes'
							else if peHTML in Self.PostExtras then
								PostData := PostData + '&postType=html'
							else
								PostData := PostData + '&postType=text';
							if peEmoticons in Self.PostExtras then
								PostData := PostData + '&emoticons=on';
							if peAutoLinks in Self.PostExtras then
								PostData := PostData + '&convertLinks=on';
							if Self.EmailReplies then
								PostData := PostData + '&emailReplies=on';
							if Assigned (Topic) then begin
								PostData := PostData + '&' + ExtractURLParams (Topic.RelURL);
								if EditedPost.PostExists then begin
									DownloadURL := URL + '.editMessageFromWeb?' + ExtractURLParams (Topic.RelURL);
									if EditedPost.ReplyIndex > 0 then begin
										PostData := PostData + '&index=' + IntToStr (ReplyIndex);
										DownloadURL := DownloadURL + '&index=' + IntToStr (ReplyIndex);
									end;
								end else
									DownloadURL := URL + '.addReplyFromWeb?' + ExtractURLParams (Topic.RelURL);
							end else
								DownloadURL := URL + '.addTopicFromWeb';
							DownloadTopic := Topic;
							PageURLs := '';
							if Assigned (Topic) then begin
								if PostExists then
									CurrentAction := raEditPost
								else
									CurrentAction := raPostReply;
							end else
								CurrentAction := raPostTopic;
							SendCommand (ExtractHost (DownloadURL), 'POST', DownloadURL, GetCookieHeader + 'Content-Length: ' + IntToStr (Length (PostData)) + #13#10'Content-Type: application/x-www-form-urlencoded'#13#10, PostData);
							CanUpdate := True;
						end;
					end;
				end else
					ShowDefaultMessageBox ('A message must contain a title and body.', 'Error', mtProgramError);
			end;
	end else begin
		EditedPost := TGUIPost.Create (Self, ItemParent);
		with EditedPost do begin
			Row.Height := Row.Height * 2;
			UserIDLabel.Caption := MainForm.GetAlias (MainForm.OwnID);
			PostTimeLabel.Caption := FormatDateTime (StdShortDateFormat + ' ' + StdShortTimeFormat, Now);
			Title.ReadOnly := False;
			if Copy (LastTitle, 1, Length ('Re: ')) <> 'Re: ' then
				LastTitle := 'Re: ' + LastTitle;
			Title.Text := LastTitle;
			Memo.ReadOnly := False;
			if Assigned (LastSplitter) then begin
				Row.Top := LastSplitter.Top + LastSplitter.Height;
				Splitter.Top := Row.Top + Row.Height;
			end;
			Insert (ItemParent);
			CreatePostIcon;
			if PostIcon.TabOrder < Title.TabOrder then
				PostIcon.TabOrder := Title.TabOrder;
			if Memo.TabOrder < PostIcon.TabOrder then
				Memo.TabOrder := PostIcon.TabOrder;
			CreatePostButton (AddReplyButtonClick);
			try
				ActiveControl := Title;
			except end;
			LastSplitter := Splitter;
			ScrollDown;
		end;
		EditingPost := True;
	end;
end;

procedure TTopicForm.SettingsButtonClick(Sender: TObject);
begin
	with TPostSettingsForm.Create (Self) do try
		UseEZCodesCheckBox.Checked := peEZCodes in PostExtras;
		UseEmoticonsCheckBox.Checked := peEmoticons in PostExtras;
		UseLinkConversionCheckBox.Checked := peAutoLinks in PostExtras;
		UseHTMLCheckBox.Checked := peHTML in PostExtras;
		EmailRepliesCheckBox.Checked := EmailReplies;
		if ShowModal = mrOK then begin
			PostExtras := [];
			if UseEZCodesCheckBox.Checked then
				Include (PostExtras, peEZCodes);
			if UseEmoticonsCheckBox.Checked then
				Include (PostExtras, peEmoticons);
			if UseLinkConversionCheckBox.Checked then
				Include (PostExtras, peAutoLinks);
			if UseHTMLCheckBox.Checked then
				Include (PostExtras, peHTML);
			EmailReplies := EmailRepliesCheckBox.Checked;
		end;
	finally
		Free;
	end;
end;

procedure TTopicForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	if Assigned (Topic) then begin
		Topic.TopicForm := nil;
		if EditingPost and Assigned (EditedPost) then
			with EditedPost do
				if not EditedPost.PostExists then begin
					with TTopicContents (Topic.TopicContents) do begin
						NewPost := Memo.Text;
						NewPostTitle := Title.Text;
					end;
		end;
	end;
	if WindowState <> wsMaximized then begin
		MainForm.TopicWinLeft := Left;
		MainForm.TopicWinTop := Top;
		MainForm.TopicWinWidth := Width;
		MainForm.TopicWinHeight := Height;
	end;
end;

procedure TTopicForm.EditButtonClick(Sender: TObject);
begin
	MainForm.CheckForCookie;
	if EditingPost and Assigned (EditedPost) then begin
		with EditedPost, MainForm do begin
			DownloadURL := URL + '.showEditScreen?' + ExtractURLParams (Topic.RelURL);
			if EditedPost.ReplyIndex > 0 then
				DownloadURL := DownloadURL + '&index=' + IntToStr (ReplyIndex);
			DownloadTopic := Topic;
			PageURLs := '';
			CurrentAction := raReloadPost;
			SendCommand (ExtractHost (DownloadURL), 'GET', DownloadURL, GetCookieHeader);
		end;
	end else begin
		EditedPost := TGUIPost ((Sender as TButton).Tag);
		PostExtras := EditedPost.PostExtras;
		with EditedPost do begin
			Row.Height := Row.Height * 3 div 2;
			Title.ReadOnly := False;
			Memo.ReadOnly := False;
			Memo.Color := clWindow;
			CreatePostIcon;
			if PostIcon.TabOrder < Title.TabOrder then
				PostIcon.TabOrder := Title.TabOrder;
			if Memo.TabOrder < PostIcon.TabOrder then
				Memo.TabOrder := PostIcon.TabOrder;
			CreatePostButton (AddReplyButtonClick);
			if Assigned (EditButton) then
				EditButton.Caption := 'Reload';
			ActiveControl := Memo;
		end;
		EditingPost := True;
	end;
end;

procedure TTopicForm.DeleteButtonClick(Sender: TObject);
var
	PostData: string;
	DelTopic: TTopic;
begin
	MainForm.CheckForCookie;
	DelTopic := nil;
	with TGUIPost ((Sender as TButton).Tag) do begin
		if Assigned (Topic) and PostExists then begin
			if ReplyIndex > 0 then begin
				if ShowDefaultMessageBox ('Are you sure you want to delete this post?', 'Confirm Deletion', mtQuestion) = idYes then
					with MainForm do begin
						PostData := ExtractURLParams (Topic.RelURL) + '&index=' + IntToStr (ReplyIndex);
						DownloadURL := URL + '.deleteSingle?' + ExtractURLParams (Topic.RelURL) + '&index=' + IntToStr (ReplyIndex);
						DownloadTopic := Topic;
						PageURLs := '';
						CurrentAction := raDeletePost;
						SendCommand (ExtractHost (DownloadURL), 'POST', DownloadURL, GetCookieHeader + 'Content-Length: ' + IntToStr (Length (PostData)) + #13#10'Content-Type: application/x-www-form-urlencoded'#13#10, PostData);
						Topic.Replies := Topic.Replies - 1;
						Topic.ReadReplies := Topic.ReadReplies - 1;
						Topic.LastComment := '';
						Topic.LatestChange := Topic.Started;
					end;
			end else begin
				if ShowDefaultMessageBox ('Are you sure you want to delete this entire topic?', 'Confirm Deletion', mtQuestion) = idYes then
					with MainForm do begin
						PostData := ExtractURLParams (Topic.RelURL);
						DownloadURL := URL + '.handleThreadControlAction?' + ExtractURLParams (Topic.RelURL) + '&threadControlAction=delete';
						DownloadTopic := nil;
						PageURLs := '';
						CurrentAction := raDeleteTopic;
						SendCommand (ExtractHost (DownloadURL), 'POST', DownloadURL, GetCookieHeader + 'Content-Length: ' + IntToStr (Length (PostData)) + #13#10'Content-Type: application/x-www-form-urlencoded'#13#10, PostData);
						DelTopic := Topic;
					end;
			end;
		end;
	end;
	if Assigned (DelTopic) then begin
		Topic := nil;
		MainForm.ExecProc (DelTopic.Free);
		MainForm.ExecProc (MainForm.SaveTopics);
		Self.Close;
	end;
end;

procedure TTopicForm.MoveUpButtonClick(Sender: TObject);
var
	Sel: TListItem;
begin
	if Assigned (Topic) then begin
		Topic.TopicForm := nil;
		if EditingPost and Assigned (EditedPost) then
			with EditedPost do
				if not EditedPost.PostExists then begin
					with TTopicContents (Topic.TopicContents) do begin
						NewPost := Memo.Text;
						NewPostTitle := Title.Text;
					end;
		end;
	end;
	CanUpdate := True;
	Topic := nil;
	with MainForm, TopicList do
		if Assigned (Selected) then begin
			Sel := Selected;
			with Sel do begin
				if (Index - 1 >= 0) and (Index - 1 < Items.Count) then
					TopicList.Selected := Items.Item [Index - 1];
				Selected := False;
			end;
			if Assigned (Selected) then
				with Selected do
					if Assigned (Data) then
						with TTopic (Data) do begin
							TopicForm := Self;
							Show;
						end;
		end;
end;

procedure TTopicForm.MoveDownButtonClick(Sender: TObject);
var
	Sel: TListItem;
begin
	if Assigned (Topic) then begin
		Topic.TopicForm := nil;
		if EditingPost and Assigned (EditedPost) then
			with EditedPost do
				if not EditedPost.PostExists then begin
					with TTopicContents (Topic.TopicContents) do begin
						NewPost := Memo.Text;
						NewPostTitle := Title.Text;
					end;
		end;
	end;
	CanUpdate := True;
	Topic := nil;
	with MainForm, TopicList do
		if Assigned (Selected) then begin
			Sel := Selected;
			with Sel do begin
				if (Index + 1 >= 0) and (Index + 1 < Items.Count) then
					TopicList.Selected := Items.Item [Index + 1];
				Selected := False;
			end;
			if Assigned (Selected) then
				with Selected do
					if Assigned (Data) then
						with TTopic (Data) do begin
							TopicForm := Self;
							Show;
						end;
		end;
end;

procedure TTopicForm.TopicLinkClick(Sender: TObject);
begin
	if Sender is TURLLabel then
		with TURLLabel (Sender) do
			if Tag <> 0 then begin
				FChangeTopic := TTopic (Tag);
				with TTimer.Create (Self) do begin
					Interval := 100;
					OnTimer := ChangeTopic;
					Enabled := True;
				end;
			end;
end;

procedure TTopicForm.ChangeTopic(Sender: TObject);
begin
	if Assigned (Sender) and (Sender is TTimer) then
		TTimer(Sender).Free;
	if Assigned (FChangeTopic) then begin
		Topic.TopicForm := nil;
		Topic := nil;
		with FChangeTopic do begin
			TopicForm := Self;
			Show;
		end;
	end;
end;

end.
